/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload;

import java.io.*;
import java.net.*;
import java.util.*;

import java.awt.*;
import javax.swing.*;

/**
 * The Reload UI Manager.
 * This takes care of general application UI duties such as setting the L&F
 * or getting Icons and Colours etc.
 *
 * @author Phillip Beauvoir
 * @version 2003-08-04
 */
public class ReloadUIManager {

    // ========================================================================
    // ============================== ICONS ===================================
    // ========================================================================
    public static String ICON_PATH = "/resources/images/";

    public static String ICON_ADL = "adl.gif";
    public static String ICON_APP16 = "app16.gif";
    public static String ICON_APP32 = "app32.gif";
    public static String ICON_COPY = "copy.gif";
    public static String ICON_CP = "cp.gif";
    public static String ICON_CPSCORM = "cp_scorm.gif";
    public static String ICON_CUT = "cut.gif";
    public static String ICON_DELETE = "delete.gif";
    public static String ICON_DEPENDENCY = "dependency.gif";
    public static String ICON_DOWN = "down.gif";
    public static String ICON_FILE = "file.gif";
    public static String ICON_FOLDER_CLOSED = "folder_closed.gif";
    public static String ICON_FOLDER_OPEN = "folder_open.gif";
    public static String ICON_IMPORT = "import.gif";
    public static String ICON_ITEM = "item.gif";
    public static String ICON_ITEMSCORM = "item_scorm.gif";
    public static String ICON_JUMP = "jump.gif";
    public static String ICON_LEFT = "left.gif";
    public static String ICON_MD = "md.gif";

    /**
	 * Tina Manoharan
	 * 2003-08-11
 	*/
    public static String ICON_VX = "vx.gif";
    public static String ICON_VXLEAF = "vx-leaf.gif";
    public static String ICON_VXFOLDER = "vx-folder.gif";
    public static String ICON_VXBROWSE = "vx-browse.gif";

    public static String ICON_MDLEAF = "md-leaf.gif";
    public static String ICON_MDFOLDER = "md-folder.gif";
    public static String ICON_ORGS = "orgs.gif";
    public static String ICON_ORG = "org.gif";
    public static String ICON_NEW = "new.gif";
    public static String ICON_OPEN = "open.gif";
    public static String ICON_PASTE = "paste.gif";
    public static String ICON_REDO = "redo.gif";
    public static String ICON_REFRESH = "refresh.gif";
    public static String ICON_RESOURCE = "resource.gif";
    public static String ICON_RESOURCES = "resources.gif";
    public static String ICON_SAVE = "save.gif";
    public static String ICON_TITLE = "title.gif";
    public static String ICON_UNDO = "undo.gif";
    public static String ICON_UP = "up.gif";
    public static String ICON_VIEWCP = "viewcp.gif";
    public static String ICON_VIEWFILE = "viewfile.gif";
    public static String ICON_ZIP = "zip.gif";

    public static String IMAGE_SPLASH = "splash.gif";
    public static String IMAGE_EASTER_EGG = "easter_egg.jpg";


    // =========================================================================
    // HELP
    // =========================================================================
    public static File getHelpFile() {
        return new File(System.getProperty("user.dir"), "/help/helpindex.html");
    }


    // ========================================================================
    // ============================== CURSORS =================================
    // ========================================================================
    public static Cursor TEXT_CURSOR = Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR);
    public static Cursor WAIT_CURSOR = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
    public static Cursor DEFAULT_CURSOR = Cursor.getDefaultCursor();


    // ========================================================================
    // =============================== FONTS ==================================
    // ========================================================================
    public static Font plainFont11 = new Font("SansSerif", Font.PLAIN, 11);
    public static Font boldFont11 = new Font("SansSerif", Font.BOLD, 11);
    public static Font italicFont11 = new Font("SansSerif", Font.ITALIC, 11);
    public static Font bold_italicFont11 = new Font("SansSerif", Font.ITALIC + Font.BOLD, 11);
    public static Font plainFont12 = new Font("SansSerif", Font.PLAIN, 12);
    public static Font boldFont12 = new Font("SansSerif", Font.BOLD, 12);
    public static Font italicFont12 = new Font("SansSerif", Font.ITALIC, 12);
    public static Font bold_italicFont12 = new Font("SansSerif", Font.ITALIC + Font.BOLD, 12);
    public static Font plainFont13 = new Font("SansSerif", Font.PLAIN, 13);
    public static Font boldFont13 = new Font("SansSerif", Font.BOLD, 13);
    public static Font italicFont13 = new Font("SansSerif", Font.ITALIC, 13);
    public static Font bold_italicFont13 = new Font("SansSerif", Font.ITALIC + Font.BOLD, 13);
    public static Font plainFont14 = new Font("SansSerif", Font.PLAIN, 14);
    public static Font boldFont14 = new Font("SansSerif", Font.BOLD, 14);

    // =========================================================================
    // LOOK AND FEEL
    // =========================================================================


    /**
     * Do some UI Defaults stuff for the System
     */
    public static void setUIDefaults() {
        // APPLE MAC STUFF
        // See http://developer.apple.com/techpubs/macosx/ReleaseNotes/java141/system_properties/index.html
        // If you are using the Aqua look and feel, this puts Swing menus in the Mac OS X menu bar.
        // Note that JMenuBars in JDialogs are not moved to the Mac OS X menu bar.
        System.setProperty("apple.laf.useScreenMenuBar", "true");
        // Sets the application name that is displayed in the application menu and in the Dock.
        System.setProperty("com.apple.mrj.application.apple.menu.about.name", ReloadEditor.APP_NAME);
        // Most native Mac OS X windows have a resize control in the bottom right corner.
        // By default, Java application windows that use the Aqua look and feel have the functionality of this control,
        // but there is no user interface cue that it is there. This flag causes the resize control to be displayed.
        System.setProperty("apple.awt.showGrowBox", "true");
    }

    /**
     * Set The Look and Feel to a class name, these will have to be installed
     * ones, or inbuilt ones.
     * @param className
     */
    public static void setLookAndFeel(String className) {
        try {
            UIManager.setLookAndFeel(className);
            updateComponentTreeUI();
        }
        catch(Exception ex) {
            // Try again with system l&f
            try {
                String lf = UIManager.getSystemLookAndFeelClassName();
                UIManager.setLookAndFeel(lf);
                updateComponentTreeUI();
            }
            catch(Exception ex1) {
                if(ReloadEditor.DEBUG) System.out.println("Error loading L&F: " + ex);
            }
        }
    }

    /**
     * Update our JFrames
     */
    private static void updateComponentTreeUI() {
        SwingUtilities.updateComponentTreeUI(ReloadEditor.getInstance());
        SwingUtilities.updateComponentTreeUI(ReloadStatusWindow.getInstance());
    }

    // =========================================================================
    // ICONS
    // =========================================================================

    /**
     * A collection of Icons
     */
    private static Hashtable icons = new Hashtable();

    /**
     * Get an Icon
     * @param iconName The name of the Icon as per those defined in ReloadConstants
     * @return The ImageIcon or null if not found
     */
    public static ImageIcon getIcon(String iconName) {
         try {
            String fullIconName = ICON_PATH + iconName;
            ImageIcon imageIcon = (ImageIcon)icons.get(fullIconName);
            if(imageIcon == null) {
                URL url = icons.getClass().getResource(fullIconName);
                if(url == null) return null;
                imageIcon = new ImageIcon(url);
                icons.put(fullIconName, imageIcon);
            }

            return imageIcon;
         }
         catch(Exception ex) {
            return null;
         }
    }

    /**
     * Gets a resource image
     * @param imageName The name of the Image as per those defined in ReloadConstants
     * @return The Image or null if not found
     */
    public static Image getImage(String imageName) {
        ImageIcon imageIcon = getIcon(imageName);
        if(imageIcon != null) return imageIcon.getImage();
        else return null;
    }
}