/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.contentpackaging;

import java.io.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.schema.*;

/**
 * The Abstract Content Package Class
 *
 * @author Phillip Beauvoir
 * @version 2003-07-01
 */
public abstract class AbstractContentPackage
extends ReloadDocument
{
    /**
     * Default Constructor
     */
    protected AbstractContentPackage() {
        super();
    }

    /**
     * Constructor
     */
    protected AbstractContentPackage(File file, Document doc, ReloadSchemaController schemaController) {
        super(file, doc, schemaController);
    }

    /**
     * Import (aggregate) another Package into this Content Package at manifestElement
     */
    public abstract Element importManifest(File manifestFile, Element manifestElement, String base) throws ReloadException;

    /**
     * Export (disagggregate) manifestElement to new Content Package
     */
    public abstract boolean exportManifest(Element manifestElement, File targetFolder) throws ReloadException;


    /**
     * @return the MD Namespace for a CP Document or null if none declared
     */
    public abstract Namespace getMetadataNamespace();

    /**
     * @return whether we have the MD root name of any embedded Metadata "lom"
     */
    public abstract boolean isMetadataRoot(String name);

    /**
     * @return whether we have the "metadata" element
     */
    public abstract boolean isMetadataElement(String name);

}
