/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.contentpackaging;

import java.io.*;

import javax.swing.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.util.*;

/**
 * The CP ReloadSchema Controller
 *
 * @author Phillip Beauvoir
 * @version 2003-08-04
 */
public class ContentPackageSchemaController
extends ReloadSchemaController
{

    /**
     * Constructor
     */
    public ContentPackageSchemaController(File fileSchema, File fileHelper, File fileVocab) {
        super(fileSchema, fileHelper, fileVocab);
    }

    /**
     * @return the Default ContentPackageSchemaController
     */
    public static ContentPackageSchemaController getDefaultSchemaController() {
        return (ContentPackageSchemaController)ReloadDocumentHandler.getReloadSchemaController(ReloadDocumentHandler.IMSCP_113);
    }

    /**
     * @return the Root element name
     */
    public String getRootElementName() {
        return ContentPackage.MANIFEST;
    }

    /**
     * @return true if nameSpaceURI is in our list of accepted Namespace URIs
     */
    public static boolean isAcceptedNamespaceURI(String nameSpaceURI) {
        for(int i = 0; i < ReloadDocumentHandler.IMSCP_NAMESPACE_URIS.length; i++) {
            if(nameSpaceURI.equals(ReloadDocumentHandler.IMSCP_NAMESPACE_URIS[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * Get The Leaf Icon
     */
    public Icon getLeafIcon(Element element) {
        // Only want to only display icons for our Namespace
        if(!isAcceptedNamespaceURI(element.getNamespace().getURI()))
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_MDLEAF);

        String elementName = element.getName();

        // Set some icons - HARD CODED FOR NOW
        if(elementName.equals(ContentPackage.MANIFEST)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_CP);
        }
        else if(elementName.equals(ContentPackage.ORGANIZATIONS)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_ORGS);
        }
        else if(elementName.equals(ContentPackage.ORGANIZATION)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_ORG);
        }
        else if(elementName.equals(ContentPackage.ITEM)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_ITEM);
        }
        else if(elementName.equals(ContentPackage.FILE)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_FILE);
        }
        else if(elementName.equals(ContentPackage.METADATA)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_MD);
        }
        else if(elementName.equals(ContentPackage.RESOURCES)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_RESOURCES);
        }
        else if(elementName.equals(ContentPackage.RESOURCE)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_RESOURCE);
        }
        //else if(elementName.equals(ContentPackage.TITLE)) {
        //    return ReloadUIManager.getIcon(ReloadUIManager.ICON_TITLE);
        //}
        else if(elementName.equals(ContentPackage.DEPENDENCY)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_DEPENDENCY);
        }
        else {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_MDLEAF);
        }
    }

    /**
     * Get The Leaf Icon
     */
    public Icon getLeafIcon(ReloadSchemaElement schemaElement) {
        // Only want to only display icons for our Namespace
        if(!isAcceptedNamespaceURI(schemaElement.getNamespace().getURI()))
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_MDLEAF);

        String elementName = schemaElement.getName();

        // Set some icons - HARD CODED FOR NOW
        if(elementName.equals(ContentPackage.MANIFEST)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_CP);
        }
        else if(elementName.equals(ContentPackage.ORGANIZATIONS)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_ORGS);
        }
        else if(elementName.equals(ContentPackage.ORGANIZATION)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_ORG);
        }
        else if(elementName.equals(ContentPackage.ITEM)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_ITEM);
        }
        else if(elementName.equals(ContentPackage.FILE)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_FILE);
        }
        else if(elementName.equals(ContentPackage.METADATA)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_MD);
        }
        else if(elementName.equals(ContentPackage.RESOURCES)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_RESOURCES);
        }
        else if(elementName.equals(ContentPackage.RESOURCE)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_RESOURCE);
        }
        //else if(elementName.equals(ContentPackage.TITLE)) {
        //    return ReloadUIManager.getIcon(ReloadUIManager.ICON_TITLE);
        //}
        else if(elementName.equals(ContentPackage.DEPENDENCY)) {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_DEPENDENCY);
        }
        else {
            return ReloadUIManager.getIcon(ReloadUIManager.ICON_MDLEAF);
        }
    }

    /**
     * Get The Open Icon
     */
    public Icon getOpenIcon(Element element) {
        return getLeafIcon(element);
    }

    /**
     * Get The Open Icon
     */
    public Icon getOpenIcon(ReloadSchemaElement schemaElement) {
        return getLeafIcon(schemaElement);
    }

    /**
     * Get The Closed Icon
     */
    public Icon getClosedIcon(Element element) {
        return getLeafIcon(element);
    }

    /**
     * Get The Closed Icon
     */
    public Icon getClosedIcon(ReloadSchemaElement schemaElement) {
        return getLeafIcon(schemaElement);
    }

    /**
     * Copy the Default Schema Files to the Project Folder
     */
    public void copySchemaFilesToFolder(File projectFolder) throws ReloadException {
        try {
            // Copy default Metadata Schema Files
            ReloadUtils.copyFolder(new File(ReloadDocumentHandler.SCHEMASHIPPEDFOLDER, ReloadDocumentHandler.IMSMD_122), projectFolder, null);
            // Copy default CP schema Files
            ReloadUtils.copyFolder(new File(ReloadDocumentHandler.SCHEMASHIPPEDFOLDER, ReloadDocumentHandler.IMSCP_113), projectFolder, null);
        }
        catch(IOException ex) {
            throw new ReloadException(ex);
        }
    }
}