/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */


package uk.ac.reload.contentpackaging;

import java.io.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.metadata.*;
import uk.ac.reload.util.*;



/**
 * The SCORM Content Package Editor Frame
 *
 *
 * @author Phillip Beauvoir
 * @version 2003-07-25
 */
public class SCORM1_2PackageEditor extends ContentPackageEditor {

    /**
     * Constructor
     */
    public SCORM1_2PackageEditor() {
        super();
        setFrameIcon(ReloadUIManager.ICON_CPSCORM);
        setTitle("SCORM Package");
    }

    /**
     * Over-ride this to set up some stuff when a new SCORM CP is loaded
     */
    protected void setContentPackage(ContentPackage contentPackage) {
        super.setContentPackage(contentPackage);
        setTitle("SCORM Package - " + contentPackage.getProjectName());
    }

    /**
     * Load an existing SCORM Content Package
     * @throws ReloadException
     */
    public void setDocument(File file, Document doc, SCORM1_2SchemaController schemaController) throws ReloadException {
        _contentPackage = new SCORM1_2Package(file, doc, schemaController);
        setContentPackage(_contentPackage);
    }

    /**
     * Start a new CP Project with a new imsmanifest.xml
     */
    public void newContentPackage(File projectFolder) throws ReloadException {
        _contentPackage = new SCORM1_2Package();

        // Default CP Schema Controller
        SCORM1_2SchemaController cpController = SCORM1_2SchemaController.getDefaultSCORMSchemaController();

        // Default MD Schema Controller 1.2.1
        MetadataSchemaController mdController = SCORM1_2SchemaController.getDefaultMetadataSchemaController();

        _contentPackage.newContentPackage(projectFolder, cpController, mdController);

        setContentPackage(_contentPackage);
    }

}