/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */


package uk.ac.reload.dom;

import java.beans.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.contentpackaging.*;
import uk.ac.reload.metadata.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.simplesequencing.*;
import uk.ac.reload.util.*;

/**
* Tina Manoharan
* 2003-08-12
*/
import uk.ac.reload.vocabulary.*;
import uk.ac.reload.vocabulary.importexport.*;


/**
 * This assists with loading, saving, determining versions and schema for a
 * JDOM XML Document file and the ReloadSchema associated with it plus helper methods.
 *
 * @author Phillip Beauvoir
 * @version 2003-07-29
 */
public final class ReloadDocumentHandler {
    // Schema Versions

    // MD Schema Versions
    public static String IMSMD_110 = "imsmd_110";
    public static String IMSMD_120 = "imsmd_120";
    public static String IMSMD_121 = "imsmd_121";
    public static String IMSMD_122 = "imsmd_122";

	/**
	* Tina Manoharan
	* 2003-08-11
	*/
    // VX Schema Versions
	public static String IMSVX_000 = "imsvx_000";
	public static String IMSVX_LAX_000 = "imsvx_lax_000";
	public static String IMSVX_FLAT_000 = "imsvx_flat_000";
	public static String IMSVX_HIERARCHICAL_000 = "imsvx_hierarchical_000";
	public static String IMSVX_GLOSSARY_000 = "imsvx_glossary_000";
	public static String IMSVX_THESAURUS_000 = "imsvx_thesaurus_000";

	public static String IMSVX_100 = "imsvx_100";
	public static String IMSVX_LAX_100 = "imsvx_lax_100";
	public static String IMSVX_FLAT_100 = "imsvx_flat_100";
	public static String IMSVX_HIERARCHICAL_100 = "imsvx_hierarchical_100";
	public static String IMSVX_GLOSSARY_100 = "imsvx_glossary_100";
	public static String IMSVX_THESAURUS_100 = "imsvx_thesaurus_100";

    // CP Schema Versions
    public static String IMSCP_111 = "imscp_111";
    public static String IMSCP_112 = "imscp_112";
    public static String IMSCP_113 = "imscp_113";

    // ADL SCORM Schema Versions
    public static String ADLCP_120_112 = "adlcp_120_112";
    public static String ADLCP_120_113 = "adlcp_120_113";
    public static String ADLCP_130 = "adlcp_130";

    // SS Schema Versions
    public static String IMSSS_100 = "imsss_100";

    /**
     * Schema root Folder
     */
    public static File SCHEMAROOTFOLDER = new File(System.getProperty("user.dir"), "schema");

    /**
     * Schema shipped Folder
     */
    public static File SCHEMASHIPPEDFOLDER = new File(SCHEMAROOTFOLDER, "shipped");

    /**
     * Schema model Folder
     */
    public static File SCHEMAMODELFOLDER = new File(SCHEMAROOTFOLDER, "model");

    /**
     * Helper root Folder
     */
    public static File HELPERFOLDER = new File(System.getProperty("user.dir"), "helpers");


    // =========================================================================
    // ================================ METADATA ===============================
    // =========================================================================

    // Schema files
    public static File fileSchemaMD1_2_1 = new File(new File(SCHEMAMODELFOLDER, IMSMD_121), "imsmd_rootv1p2p1.xsd");
    public static File fileSchemaMD1_2_2 = new File(new File(SCHEMAMODELFOLDER, IMSMD_122), "imsmd_v1p2p2.xsd");

    // Helper files
    public static File folderMDHelper = new File(HELPERFOLDER, "md");
    public static File fileHelperMD = new File(folderMDHelper, "imsmd_en_helper.xml");

    // Vocab files
    public static File folderMDVocab = new File(folderMDHelper, "vocab");
    public static File fileVocabMD_IMS = new File(folderMDVocab, "ims-lrm_en_vocab.xml");

    // Vdex Vocab files
    public static File folderMDVdexVocab = new File(folderMDHelper, "vdexVocab");

    // Profile file
    public static File folderMDProfile = new File(folderMDHelper, "profile");
    public static File defaultMDProfile = new File(folderMDProfile, "ims-lrm_en_profile.xml");

    // Namespace prefix
    public static String IMSMD_NAMESPACE_PREFIX = "imsmd";

    // MD Version 1.2.2
    public static String IMSMD_NAMESPACE_URI_122 = "http://www.imsglobal.org/xsd/imsmd_v1p2";          // Correct
    // MD Version 1.2.1
    public static String IMSMD_NAMESPACE_URI_121 = "http://www.imsglobal.org/xsd/imsmd_rootv1p2p1";    // Correct
    public static String IMSMD_NAMESPACE_URI_121A = "http://www.imsproject.org/xsd/imsmd_rootv1p2p1";    // Bogus
    // MD Version 1.2 Variations
    public static String IMSMD_NAMESPACE_URI_12A = "http://www.imsproject.org/xsd/imsmd_rootv1p2";     // Bogus
    public static String IMSMD_NAMESPACE_URI_12B = "http://www.imsglobal.org/xsd/imsmd_rootv1p2";      // Bogus
    public static String IMSMD_NAMESPACE_URI_12C = "http://www.imsproject.org/xsd/ims_md_rootv1p2";    // Bogus
    // MD Version 1.1 Variations
    public static String IMSMD_NAMESPACE_URI_11 = "http://www.imsproject.org/xsd/ims_md_rootv1p1";
    public static String IMSMD_NAMESPACE_URI_112 = "http://www.imsproject.org/xsd/imsmd_rootv1p1p2";   // Bogus

    // Enumeration of Supported MD Namespace URIs
    public static String[] IMSMD_NAMESPACE_URIS = {
        IMSMD_NAMESPACE_URI_122,
        IMSMD_NAMESPACE_URI_121
    };


    /**
	 * Tina Manoharan
	 * 2003-08-13
 	*/
    // =========================================================================
    // ============================== IMS VOCABULARY ===========================
    // =========================================================================

    // Schema files
    public static File fileSchemaVX_LAX_1_0_0 = new File(new File(SCHEMAMODELFOLDER, IMSVX_100), ReloadPropertiesManager.getString("vx", "imsvx_lax_100"));
    public static File fileSchemaVX_FLAT_1_0_0 = new File(new File(SCHEMAMODELFOLDER, IMSVX_100), ReloadPropertiesManager.getString("vx", "imsvx_flat_100"));
    public static File fileSchemaVX_HIERARCHICAL_1_0_0 = new File(new File(SCHEMAMODELFOLDER, IMSVX_100), ReloadPropertiesManager.getString("vx", "imsvx_hierarchical_100"));
    public static File fileSchemaVX_GLOSSARY_1_0_0 = new File(new File(SCHEMAMODELFOLDER, IMSVX_100), ReloadPropertiesManager.getString("vx", "imsvx_glossary_100"));
    public static File fileSchemaVX_THESAURUS_1_0_0 = new File(new File(SCHEMAMODELFOLDER, IMSVX_100), ReloadPropertiesManager.getString("vx", "imsvx_thesaurus_100"));

    // Helper files
    public static File folderVXHelper = new File(HELPERFOLDER, "vx");
    public static File fileHelperVX = new File(folderVXHelper, ReloadPropertiesManager.getString("vx", "imsvx_helper"));

    // Zthes Vocab files
    public static File folderMDZthesVocab = new File(folderMDHelper, "zthesVocab");

    // Other Vocab files
    public static File folderMDOtherVocab = new File(folderMDHelper, "otherVocab");

    // Stylesheet files
    public static File folderStylesheets = new File(HELPERFOLDER, "stylesheets");
    public static File VDEX_BROWSER_STYLESHEET = new File(folderStylesheets, "VdexBrowserStyle.css");
    public static File ZTHES_TO_VDEX_STYLESHEET = new File(folderStylesheets, "zthes_to_vdex.xsl");
    public static File VDEX_TO_ZTHES_STYLESHEET = new File(folderStylesheets, "vdex_to_zthes.xsl");
    public static File VDEX_TO_TEMP_STYLESHEET = new File(folderStylesheets, "vdex_to_temp.xsl");

    // Vocab files
    public static File folderVXVocab = new File(folderVXHelper, "vocab");
    public static File fileVocabVX = new File(folderVXVocab, ReloadPropertiesManager.getString("vx", "imsvx_vocab"));

    // Profile file
    public static File folderVXProfile = new File(folderVXHelper, "profile");
    public static File defaultVXProfile = new File(folderVXProfile, ReloadPropertiesManager.getString("vx", "imsvx_lax_profile"));
    public static File fileProfileVX_LAX = new File(folderVXProfile, ReloadPropertiesManager.getString("vx", "imsvx_lax_profile"));
    public static File fileProfileVX_FLAT = new File(folderVXProfile, ReloadPropertiesManager.getString("vx", "imsvx_flat_profile"));
    public static File fileProfileVX_HIERARCHICAL = new File(folderVXProfile, ReloadPropertiesManager.getString("vx", "imsvx_hierarchical_profile"));
    public static File fileProfileVX_GLOSSARY = new File(folderVXProfile, ReloadPropertiesManager.getString("vx", "imsvx_glossary_profile"));
    public static File fileProfileVX_THESAURUS = new File(folderVXProfile, ReloadPropertiesManager.getString("vx", "imsvx_thesaurus_profile"));

    // Namespace prefix
    public static String IMSVX_NAMESPACE_PREFIX = "imsvdex";

    // VX Version 0.0.0
    public static String IMSVX_NAMESPACE_URI_000 = ReloadPropertiesManager.getString("vx", "imsvx_namespace_uri_000"); // Bogus
    // VX Version 1.0.0
    public static String IMSVX_NAMESPACE_URI_100 = ReloadPropertiesManager.getString("vx", "imsvx_namespace_uri_100"); // Correct

    // Enumeration of Supported VX Namespace URIs
    public static String[] IMSVX_NAMESPACE_URIS = {
        IMSVX_NAMESPACE_URI_100
    };


    // =========================================================================
    // ================================ IMS CP =================================
    // =========================================================================

    // Schema files
    public static File fileSchemaCP1_1_2 = new File(new File(SCHEMAMODELFOLDER, IMSCP_112), "imscp_rootv1p1p2.xsd");
    public static File fileSchemaCP1_1_3 = new File(new File(SCHEMAMODELFOLDER, IMSCP_113), "imscp_v1p1.xsd");

    // Helper files
    public static File folderCPHelper = new File(HELPERFOLDER, "cp");
    public static File fileHelperCP = new File(folderCPHelper, "imscp_en_helper.xml");

    // Vocab files
    public static File folderCPVocab = new File(folderCPHelper, "vocab");
    public static File fileVocabCP = new File(folderCPVocab, "imscp_en_vocab.xml");

    // Namespace prefix
    public static String IMSCP_NAMESPACE_PREFIX = "imscp";

    // CP Version 1.1.3
    public static String IMSCP_NAMESPACE_URI_113 = "http://www.imsglobal.org/xsd/imscp_v1p1";           // Correct
    public static String IMSCP_NAMESPACE_URI_113A = "http://www.imsproject.org/xsd/imscp_rootv1p1p3";   // Bogus
    // CP Version 1.1.2
    public static String IMSCP_NAMESPACE_URI_112 = "http://www.imsproject.org/xsd/imscp_rootv1p1p2";    // Correct
    public static String IMSCP_NAMESPACE_URI_112A = "http://www.imsglobal.org/xsd/imscp_rootv1p1p2";    // Bogus
    // CP Version 1.1.x Variations
    public static String IMSCP_NAMESPACE_URI_111A = "http://www.imsglobal.org/xsd/imscproot_v1p1";      // Bogus
    public static String IMSCP_NAMESPACE_URI_111B = "http://www.imsglobal.org/xsd/imscp_rootv1p1";      // Bogus
    public static String IMSCP_NAMESPACE_URI_111C = "http://www.imsproject.org/xsd/ims_cp_rootv1p1";    // Bogus

    // Enumeration of Supported CP Namespace URIs
    public static String[] IMSCP_NAMESPACE_URIS = {
        IMSCP_NAMESPACE_URI_113,
        IMSCP_NAMESPACE_URI_112
    };

    // =========================================================================
    // ================================ SCORM =================================
    // =========================================================================

    // Schema files
    public static File fileSchemaSCORM1_2_IMCP1_1_2 = new File(new File(SCHEMAMODELFOLDER, ADLCP_120_112), "imscp_rootv1p1p2.xsd");
    public static File fileSchemaSCORM1_2_IMCP1_1_3 = new File(new File(SCHEMAMODELFOLDER, ADLCP_120_113), "imscp_v1p1.xsd");

    // Helper files
    public static File fileHelperSCORM1_2 = new File(folderCPHelper, "scorm1_2_en_helper.xml");

    // Vocab files
    public static File fileVocabSCORM1_2 = new File(folderCPVocab, "scorm1_2_en_vocab.xml");

    // Namespace prefix
    public static String ADLCP_NAMESPACE_PREFIX = "adlcp";

    // ADL SCORM Version 1.2
    public static String ADLCP_NAMESPACE_URI_12 = "http://www.adlnet.org/xsd/adlcp_rootv1p2";
    public static Namespace ADLCP_NAMESPACE_12 = Namespace.getNamespace("adlcp", ADLCP_NAMESPACE_URI_12);

    // Enumeration of ADL SCORM Namespace URIs
    public static String[] ADLCP_NAMESPACE_URIS = {
        ADLCP_NAMESPACE_URI_12
    };

    // =========================================================================
    // ==================== SIMPLE SEQUENCING ==================================
    // =========================================================================

    // Schema files
    public static File fileSchemaSS1_0 = new File(new File(SCHEMAMODELFOLDER, IMSSS_100), "imsss_v1p0.xsd");

    // Helper files
    public static File folderSSHelper = new File(HELPERFOLDER, "ss");
    public static File fileHelperSS = new File(folderSSHelper, "imsss_en_helper.xml");

    // Vocab files
    public static File folderSSVocab = new File(folderSSHelper, "vocab");
    public static File fileVocabSS = new File(folderSSVocab, "imsss_en_vocab.xml");

    // Namespace prefix
    public static String IMSSS_NAMESPACE_PREFIX = "imsss";

    // SS Version 1.0
    public static String IMSSS_NAMESPACE_URI_10 = "http://www.imsglobal.org/xsd/imsss";

    // Enumeration of SS Namespace URIs
    public static String[] IMSSS_NAMESPACE_URIS = {
        IMSSS_NAMESPACE_URI_10
    };

    // =========================================================================
    // ========================== NAMESPACING ==================================
    // =========================================================================

    /**
     * The XSI Namespace
     */
    public static Namespace XSI_Namespace = Namespace.getNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");

    /**
     * The Old XSI Namespace
     */
    public static Namespace XSI_NamespaceOLD = Namespace.getNamespace("xsi", "http://www.w3.org/2000/10/XMLSchema-instance");

    /**
     * The schemaLocation String
     */
    public static String XSI_SchemaLocation = "schemaLocation";

    /**
     * A static table of Schema Controllers
     */
    private static Hashtable _schemaTable = new Hashtable();

    // Schema Namespace URIs mapped to Version numbers
    static String[] _namespaceURIs = new String[] {
        // MD
        IMSMD_NAMESPACE_URI_122, IMSMD_122,
        IMSMD_NAMESPACE_URI_121, IMSMD_121,
        IMSMD_NAMESPACE_URI_12A, IMSMD_120,
        IMSMD_NAMESPACE_URI_12B, IMSMD_120,
        IMSMD_NAMESPACE_URI_12C, IMSMD_120,
        // VX
        IMSVX_NAMESPACE_URI_100, IMSVX_100,
        // CP
        IMSCP_NAMESPACE_URI_113, IMSCP_113,
        IMSCP_NAMESPACE_URI_113A, IMSCP_113,
        IMSCP_NAMESPACE_URI_112, IMSCP_112,
        IMSCP_NAMESPACE_URI_112A, IMSCP_112,
        IMSCP_NAMESPACE_URI_111A, IMSCP_111,
        IMSCP_NAMESPACE_URI_111B, IMSCP_111,
        IMSCP_NAMESPACE_URI_111C, IMSCP_111,
        // SS
        IMSSS_NAMESPACE_URI_10, IMSSS_100
    };

    // Enumeration of Bogus Namespace URIs mapped to supported ones
    public static String[] BOGUS_NAMESPACE_MAPPING = {
        IMSCP_NAMESPACE_URI_113A, IMSCP_NAMESPACE_URI_113,
        IMSCP_NAMESPACE_URI_112A, IMSCP_NAMESPACE_URI_112,
        IMSCP_NAMESPACE_URI_111A, IMSCP_NAMESPACE_URI_113,
        IMSCP_NAMESPACE_URI_111B, IMSCP_NAMESPACE_URI_113,
        IMSCP_NAMESPACE_URI_111C, IMSCP_NAMESPACE_URI_113,
        IMSMD_NAMESPACE_URI_12A, IMSMD_NAMESPACE_URI_122,
        IMSMD_NAMESPACE_URI_12B, IMSMD_NAMESPACE_URI_122,
        IMSMD_NAMESPACE_URI_12C, IMSMD_NAMESPACE_URI_122,
        IMSMD_NAMESPACE_URI_11, IMSMD_NAMESPACE_URI_122,
        IMSMD_NAMESPACE_URI_121A, IMSMD_NAMESPACE_URI_121,
        IMSMD_NAMESPACE_URI_112, IMSMD_NAMESPACE_URI_121,
        IMSVX_NAMESPACE_URI_000, IMSVX_NAMESPACE_URI_100
    };





    // =========================================================================
    // ======================= SCHEMACONTROLLER HANDLING =======================
    // =========================================================================

    /**
     * @return a (root) Schema version depending on an Instance ReloadDocument
     */
    public static String getSchemaVersion(Document doc) {
        // Get the correct Namespace
        Namespace ns = getCorrectRootNamespace(doc);

        // Get root schema version
        String schemaVersion = getSchemaVersionFromNamespace(ns);

        // If it's a CP, check for ADL SCORM
        if(schemaVersion != null && schemaVersion.startsWith("imscp")) {
            Namespace nsSCORM = getDocumentNamespace(doc, ADLCP_NAMESPACE_PREFIX);
            if(nsSCORM != null) {
                if(schemaVersion.equals(IMSCP_112)) schemaVersion = ADLCP_120_112;
                else if(schemaVersion.equals(IMSCP_113)) schemaVersion = ADLCP_120_113;
            }
        }

		/**
		* Tina Manoharan
		* 2003-08-13
		*/

        // if it's a Vdex vocabulary
        if(schemaVersion != null && schemaVersion.startsWith("imsvx")) {

			schemaVersion = getSchemaProfileVersion(doc);
		}

        return schemaVersion;
    }



    /**
     * @return an appropriate ReloadSchemaController for a given Namespace in an Instance ReloadDocument
     */
    public static ReloadSchemaController getReloadSchemaController(Document doc, Namespace ns) {
        // Try to correct the Namespace
        ns = correctAdditionalNamespace(doc, ns);

        String version = getSchemaVersionFromNamespace(ns);
        return getReloadSchemaController(version);
    }


    /**
     * @return a Schema Controller depending on a Schema version
     */
    public static ReloadSchemaController getReloadSchemaController(String schemaVersion) {

        ReloadSchemaController schemaController = null;

        if(schemaVersion != null) {
            // METADATA 1.1
            if(schemaVersion.equals(IMSMD_110)) {
                showWarningMessage("This is Metadata version 1.1.  Will use version 1.2.2 Schema model.");
                return getReloadSchemaController(IMSMD_122);
            }

            // METADATA 1.2
            else if(schemaVersion.equals(IMSMD_120)) {
                showWarningMessage("This is Metadata version 1.2.  Will use version 1.2.2 Schema model.");
                return getReloadSchemaController(IMSMD_122);
            }

            // METADATA 1.2.1
            else if(schemaVersion.equals(IMSMD_121)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(IMSMD_121);
                if(schemaController == null) {

                    schemaController = new MetadataSchemaController(fileSchemaMD1_2_1, fileHelperMD, fileVocabMD_IMS);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }

            // METADATA 1.2.2
            else if(schemaVersion.equals(IMSMD_122)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(IMSMD_122);
                if(schemaController == null) {

                    schemaController = new MetadataSchemaController(fileSchemaMD1_2_2, fileHelperMD, fileVocabMD_IMS);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }

		    /**
			 * Tina Manoharan
			 * 2003-08-13
		 	*/
            // VDEX 0.0.0, ProfileType LAX
            else if(schemaVersion.equals(IMSVX_LAX_000)) {
                showWarningMessage("This is Vocabulary version 0.0.0 LAX.  Will use version 1.0.0 LAX Schema model.");
                return getReloadSchemaController(IMSVX_LAX_000);
            }

            // VDEX 0.0.0, ProfileType FLAT
            else if(schemaVersion.equals(IMSVX_FLAT_000)) {
                showWarningMessage("This is Vocabulary version 0.0.0 FLAT.  Will use version 1.0.0 FLAT Schema model.");
                return getReloadSchemaController(IMSVX_FLAT_000);
            }

            // VDEX 0.0.0, ProfileType HIERARCHICAL
            else if(schemaVersion.equals(IMSVX_HIERARCHICAL_000)) {
                showWarningMessage("This is Vocabulary version 0.0.0 HIERARCHICAL.  Will use version 1.0.0 HIERARCHICAL Schema model.");
                return getReloadSchemaController(IMSVX_HIERARCHICAL_000);
            }

            // VDEX 0.0.0, ProfileType GLOSSARY
            else if(schemaVersion.equals(IMSVX_GLOSSARY_000)) {
                showWarningMessage("This is Vocabulary version 0.0.0 GLOSSARY.  Will use version 1.0.0 GLOSSARY Schema model.");
                return getReloadSchemaController(IMSVX_GLOSSARY_000);
            }

            // VDEX 0.0.0, ProfileType THESAURUS
            else if(schemaVersion.equals(IMSVX_THESAURUS_000)) {
                showWarningMessage("This is Vocabulary version 0.0.0 THESAURUS.  Will use version 1.0.0 THESAURUS Schema model.");
                return getReloadSchemaController(IMSVX_THESAURUS_000);
            }

            // VDEX 1.0, ProfileType LAX
            else if(schemaVersion.equals(IMSVX_LAX_100)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(IMSVX_LAX_100);
                if(schemaController == null) {

                    schemaController = new VocabularySchemaController(fileSchemaVX_LAX_1_0_0, fileHelperVX, fileVocabVX);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }

            // VDEX 1.0, ProfileType FLAT
            else if(schemaVersion.equals(IMSVX_FLAT_100)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(IMSVX_FLAT_100);
                if(schemaController == null) {

                    schemaController = new VocabularySchemaController(fileSchemaVX_FLAT_1_0_0, fileHelperVX, fileVocabVX);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }

            // VDEX 1.0, ProfileType HIERARCHICAL
            else if(schemaVersion.equals(IMSVX_HIERARCHICAL_100)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(IMSVX_HIERARCHICAL_100);
                if(schemaController == null) {

                    schemaController = new VocabularySchemaController(fileSchemaVX_HIERARCHICAL_1_0_0, fileHelperVX, fileVocabVX);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }

            // VDEX 1.0, ProfileType GLOSSARY
            else if(schemaVersion.equals(IMSVX_GLOSSARY_100)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(IMSVX_GLOSSARY_100);
                if(schemaController == null) {

                    schemaController = new VocabularySchemaController(fileSchemaVX_GLOSSARY_1_0_0, fileHelperVX, fileVocabVX);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }

            // VDEX 1.0, ProfileType THESAURUS
            else if(schemaVersion.equals(IMSVX_THESAURUS_100)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(IMSVX_THESAURUS_100);
                if(schemaController == null) {

                    schemaController = new VocabularySchemaController(fileSchemaVX_THESAURUS_1_0_0, fileHelperVX, fileVocabVX);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }


            // CP 1.1.1
            else if(schemaVersion.equals(IMSCP_111)) {
                showWarningMessage("This is Content Packaging version 1.1.  Will use version 1.1.3 Schema model.");
                return getReloadSchemaController(IMSCP_113);
            }

            // CP 1.1.2
            else if(schemaVersion.equals(IMSCP_112)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(IMSCP_112);
                if(schemaController == null) {
                    schemaController = new ContentPackageSchemaController(fileSchemaCP1_1_2, fileHelperCP, fileVocabCP);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }

            // CP 1.1.3
            else if(schemaVersion.equals(IMSCP_113)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(IMSCP_113);
                if(schemaController == null) {
                    schemaController = new ContentPackageSchemaController(fileSchemaCP1_1_3, fileHelperCP, fileVocabCP);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }

            // SCORM 1.2 IMS CP 1.1.2
            else if(schemaVersion.equals(ADLCP_120_112)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(ADLCP_120_112);
                if(schemaController == null) {
                    schemaController = new SCORM1_2SchemaController(fileSchemaSCORM1_2_IMCP1_1_2, fileHelperSCORM1_2, fileVocabSCORM1_2);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }

            // SCORM 1.2 IMS CP 1.1.3
            else if(schemaVersion.equals(ADLCP_120_113)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(ADLCP_120_113);
                if(schemaController == null) {
                    schemaController = new SCORM1_2SchemaController(fileSchemaSCORM1_2_IMCP1_1_3, fileHelperSCORM1_2, fileVocabSCORM1_2);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }

            // SS 1.0
            else if(schemaVersion.equals(IMSSS_100)) {
                schemaController = (ReloadSchemaController) _schemaTable.get(IMSSS_100);
                if(schemaController == null) {
                    schemaController = new SimpleSequencingSchemaController(fileSchemaSS1_0, fileHelperSS, fileVocabSS);
                    _schemaTable.put(schemaVersion, schemaController);
                }
            }
        }

        return schemaController;
    }

    /**
     * @return a version of the Schema depending on Namespace URI
     */
    public static String getSchemaVersionFromNamespace(Namespace ns) {
        if(ns != null) {
            String nameSpaceURI = ns.getURI();

            String version = "";

            for(int i = 0; i < _namespaceURIs.length; i += 2) {
                if(nameSpaceURI.equalsIgnoreCase(_namespaceURIs[i])) {

            		version = _namespaceURIs[i + 1];

					return version;
				}
            }
        }
        return null;
    }


    /**
     * @return a (root) Schema Profile Version
     */
    public static String getSchemaProfileVersion(Document doc) {

		String schemaProfileVersion = "", profileType = "";

		Element root = doc.getRootElement();

		profileType = root.getAttributeValue("profileType");

		if(profileType.equals("lax")) schemaProfileVersion = IMSVX_LAX_100;
		else if(profileType.equals("flatTokenTerms")) schemaProfileVersion = IMSVX_FLAT_100;
		else if(profileType.equals("hierarchicalTokenTerms")) schemaProfileVersion = IMSVX_HIERARCHICAL_100;
		else if(profileType.equals("glossaryOrDictionary")) schemaProfileVersion = IMSVX_GLOSSARY_100;
		else if(profileType.equals("thesaurus")) schemaProfileVersion = IMSVX_THESAURUS_100;

        return schemaProfileVersion;
    }


    /**
     * @return a Schema Profile File
     */
    public static File getVXSchemaProfileFile(String schemaFileName) {

		if(schemaFileName.equals(fileSchemaVX_LAX_1_0_0.getName())) return fileProfileVX_LAX;
		else if(schemaFileName.equals(fileSchemaVX_FLAT_1_0_0.getName())) return fileProfileVX_FLAT;
		else if(schemaFileName.equals(fileSchemaVX_HIERARCHICAL_1_0_0.getName())) return fileProfileVX_HIERARCHICAL;
		else if(schemaFileName.equals(fileSchemaVX_GLOSSARY_1_0_0.getName())) return fileProfileVX_GLOSSARY;
		else if(schemaFileName.equals(fileSchemaVX_THESAURUS_1_0_0.getName())) return fileProfileVX_THESAURUS;

		return null;
	}


    /**
     * Check for bogus root Namespace and correct it
     */
    public static Namespace getCorrectRootNamespace(Document doc) {
        // Get the Root Namespace for the Instance Document
        Namespace ns = getDocumentNamespace(doc);
        Element element = doc.getRootElement();
        if(element != null && ns != null) {
            String nameSpaceURI = ns.getURI();
            for(int i = 0; i < BOGUS_NAMESPACE_MAPPING.length; i += 2) {
                if(nameSpaceURI.equalsIgnoreCase(BOGUS_NAMESPACE_MAPPING[i])) {
                    Namespace newNamespace = Namespace.getNamespace(BOGUS_NAMESPACE_MAPPING[i + 1]);
                    replaceNamespaces(element, ns, newNamespace);
                    if(ReloadEditor.DEBUG) System.out.println("Corrected namespace: " + ns.getURI() + " ----> " + newNamespace.getURI());
                    return newNamespace;
                }
            }
        }
        return ns;
    }

    /**
     * Check for bogus additional Namespaces and correct them
     */
    public static void correctAdditionalNamespaces(Document doc) {
        Element element = doc.getRootElement();
        // Use a copy of the list so we don't get a concurrent modification exception
        Vector v = new Vector();
        Iterator it = element.getAdditionalNamespaces().listIterator();
        while(it.hasNext()) {
            Namespace ns = (Namespace) it.next();
            v.add(ns);
        }
        it = v.iterator();
        while(it.hasNext()) {
            Namespace ns = (Namespace) it.next();
            correctAdditionalNamespace(doc, ns);
        }
    }

    /**
     * Check for bogus additional Namespace and correct it
     */
    public static Namespace correctAdditionalNamespace(Document doc, Namespace ns) {
        if(doc != null && ns != null) {
            Element element = doc.getRootElement();
            String nameSpaceURI = ns.getURI();
            for(int i = 0; i < BOGUS_NAMESPACE_MAPPING.length; i += 2) {
                if(nameSpaceURI.equalsIgnoreCase(BOGUS_NAMESPACE_MAPPING[i])) {
                    Namespace newNamespace = Namespace.getNamespace(ns.getPrefix(), BOGUS_NAMESPACE_MAPPING[i + 1]);
                    element.removeNamespaceDeclaration(ns);
                    element.addNamespaceDeclaration(newNamespace);
                    replaceNamespaces(element, ns, newNamespace);
                    if(ReloadEditor.DEBUG) System.out.println("Corrected namespace: " + ns.getURI() + " ----> " + newNamespace.getURI());
                    return newNamespace;
                }
            }
        }
        return ns;
    }

    /**
     * Set child namespaces to MD Namespace.  Will not affect other Namespaces
     * @param element
     */
    public static void replaceNamespaces(Element element, Namespace oldNamespace, Namespace newNamespace) {
        if(element.getNamespace().getURI().equalsIgnoreCase(oldNamespace.getURI()))
            element.setNamespace(newNamespace);

        Iterator children = element.getChildren().iterator();
        while(children.hasNext()) {
            Element child = (Element)children.next();
            replaceNamespaces(child, oldNamespace, newNamespace);
        }
    }


    /**
     * Replace the old "record" element with "lom" on MD
     */
    public static void correctLomElement(Element element) {
        if(element != null) {
            if(element.getName().equals("record")) {
                element.setName("lom");
                if(ReloadEditor.DEBUG) System.out.println("Corrected \"record\" ---> \"lom\" element");
            }
            Iterator children = element.getChildren().iterator();
            while(children.hasNext()) {
                Element child = (Element)children.next();
                correctLomElement(child);
            }
        }
    }

    // =========================================================================
    // ========================== DOCUMENT HANDLING ============================
    // =========================================================================

    /**
     * The Manifest name
     */
    public static final String MANIFEST_NAME = "imsmanifest.xml";

    /**
     * Open a file, determine what it is, load it, get the SchemaController
     * and open an appropriate Editor
     */
    public static void openFile() {
        // Ask for file
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml", "zip"}, "XML or Archive Files");
        File file = XMLUtils.askFileNameOpen("Open XML or Archive file", filter);
        if(file == null) return;

        // If we already have it open select and maximise it
        ReloadInternalFrame frame = ReloadInternalFrameManager.getReloadInternalFrame(file);
        if(frame != null) {
            try {
                frame.setIcon(false);
                frame.setSelected(true);
            }
            catch(PropertyVetoException ex) {}
            return;
        }

        try {
            // If user opens a CP zip file then we ask to unzip it
            if(file.getName().toLowerCase().endsWith(".zip")) {
                file = unzipContentPackage(file);
                if(file == null) return;
            }

            // Wait cursor
            ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

            // Load the Document
            Document doc = loadDocument(file);

            // OK, what is it?
            String version = getSchemaVersion(doc);

            // Unknown version
            if(version == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unknown version.  Sorry, cannot open file.");
                return;
            }

            // Get the Schema Controller
            ReloadSchemaController schemaController = getReloadSchemaController(version);

            // Oops!
            if(schemaController == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unknown version.  Sorry, cannot open file.");
                return;
            }

            // Get rid of bogus Namespaces
            correctAdditionalNamespaces(doc);

            // Get rid of old "record" elements
            correctLomElement(doc.getRootElement());

            // Now get the appropriate Editor and ReloadDocument

            // MD Editor
            if(schemaController instanceof MetadataSchemaController) {
                MetadataEditorFrame mdEditorFrame = new MetadataEditorFrame();
                mdEditorFrame.setDocument(file, doc, (MetadataSchemaController)schemaController);
                launchEditor(mdEditorFrame);
            }

			/**
			* Tina Manoharan
			* 2003-08-12
		 	*/
		 	// VX Editor
		 	if(schemaController instanceof VocabularySchemaController) {
		 		VocabularyEditorFrame vxEditorFrame = new VocabularyEditorFrame();
		 		vxEditorFrame.setDocument(file, doc, (VocabularySchemaController)schemaController);
		 		launchEditor(vxEditorFrame);
			}


            // ADL SCORM 1.2 Editor
            else if(schemaController instanceof SCORM1_2SchemaController) {
                SCORM1_2PackageEditor scormEditorFrame = new SCORM1_2PackageEditor();
                scormEditorFrame.setDocument(file, doc, (SCORM1_2SchemaController)schemaController);
                launchEditor(scormEditorFrame);
            }

            // IMSP CP Editor
            else if(schemaController instanceof ContentPackageSchemaController) {
                ContentPackageEditor cpEditorFrame = new ContentPackageEditor();
                cpEditorFrame.setDocument(file, doc, (ContentPackageSchemaController)schemaController);
                launchEditor(cpEditorFrame);
            }

        }
        catch(ReloadException ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Error opening file: " + file.getName(), "Could not open file", ex);
        }
        finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
        }
    }




    public static ReloadDocument openFile(File file) {

		try {

            // Load the Document
            Document doc = loadDocument(file);

            // OK, what is the doc?
            String version = getSchemaVersion(doc);

            // Unknown version
            if(version == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unknown version.  Sorry, cannot open file.");
                return null;
            }

            // Get the Schema Controller
            ReloadSchemaController schemaController = getReloadSchemaController(version);

            // Oops!
            if(schemaController == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unknown version.  Sorry, cannot open file.");
                return null;
            }

            // Get rid of bogus Namespaces
            correctAdditionalNamespaces(doc);

            // Now get the appropriate Editor and ReloadDocument

		 	// VX Editor
		 	if(schemaController instanceof VocabularySchemaController) {

				Vocabulary vocabulary = new Vocabulary(true);
		 		vocabulary.setDocument(file, doc, (VocabularySchemaController)schemaController);

		 		return vocabulary;
			}

        }
        catch(ReloadException ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Error opening file: " + file.getName(), "Could not open file", ex);
        }
        finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
        }

        return null;
    }




    public static void openFile2(File file) {

		try {

            // Load the Document
            Document doc = loadDocument(file);

            // OK, what is the doc?
            String version = getSchemaVersion(doc);

            // Unknown version
            if(version == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unknown version.  Sorry, cannot open file.");
                return;
            }

            // Get the Schema Controller
            ReloadSchemaController schemaController = getReloadSchemaController(version);

            // Oops!
            if(schemaController == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unknown version.  Sorry, cannot open file.");
                return;
            }

            // Get rid of bogus Namespaces
            correctAdditionalNamespaces(doc);

            // Now get the appropriate Editor and ReloadDocument

		 	// VX Editor
		 	if(schemaController instanceof VocabularySchemaController) {
		 		VocabularyEditorFrame vxEditorFrame = new VocabularyEditorFrame();
		 		vxEditorFrame.setDocument(file, doc, (VocabularySchemaController)schemaController);
		 		launchEditor(vxEditorFrame);
			}

        }
        catch(ReloadException ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Error opening file: " + file.getName(), "Could not open file", ex);
        }
        finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
        }
    }



    /**
     * @return the XML JDOM Document
     * @param file The XML IMS File to read in
     * @throws ReloadException
     */
    public static Document loadDocument(File file) throws ReloadException {
        Document doc = XMLUtils.readXMLFile(file);
        return doc;
    }

    /**
     * @return the schema location for a given Namespace in a JDOM Document or null if not found
     */
    public static String getSchemaLocation(Document doc, Namespace ns) {
        if(doc.hasRootElement() && ns != null) {
            String namespaceURI = ns.getURI();
            if(namespaceURI != null) {
                Element root = doc.getRootElement();

                // Get Schema Location
                String str = root.getAttributeValue(XSI_SchemaLocation, XSI_Namespace);
                // Try an older one
                if(str == null)
                    str = root.getAttributeValue(XSI_SchemaLocation, XSI_NamespaceOLD);

                if(str != null) {
                    int index = str.indexOf(namespaceURI);
                    if(index != -1) {
                        str = str.substring(index + namespaceURI.length());
                        StringTokenizer t = new StringTokenizer(str, " ");
                        try {
                            return t.nextToken();
                        }
                        catch(NoSuchElementException ex) {
                            return null;
                        }
                    }
                }
            }
        }

        return null;
    }

    /**
     * @return the root Namespace of the Document or null if not found
     */
    public static Namespace getDocumentNamespace(Document doc) {
        Namespace ns = null;
        if(doc.hasRootElement()) ns = doc.getRootElement().getNamespace();
        return ns;
    }

    /**
     * @return a given Namespace of the Document given its prefix or null if not found
     */
    public static Namespace getDocumentNamespace(Document doc, String prefix) {
        Namespace ns = null;
        if(doc.hasRootElement()) ns = doc.getRootElement().getNamespace(prefix);
        return ns;
    }


    /**
     * Dialogue routine to allow user to import a new Zthes vocabulary as Vdex
     */
    public static void importZthesVocabulary(){

        // Ask for file
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "XML Files");
        File zthesFile = XMLUtils.askFileNameOpenUsingDirName("Open XML Zthes Vocabulary file", filter, folderMDZthesVocab);
        if(zthesFile == null) return;

        try {
            // Call routine
            showWarningMessage("Possible loss of information when importing Zthes to Vdex");

            ZthesImporter zthesImporter = new ZthesImporter(zthesFile);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Import Zthes Vocabulary as Vdex", "Error importing", ex);
        }

    }

    /**
     * Dialogue routine to allow user to import other vocabulary as Vdex
     */
    public static void importOtherVocabulary(){

        // Ask for file
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "XML Files");
        File otherFile = XMLUtils.askFileNameOpenUsingDirName("Open XML Vocabulary file", filter, folderMDOtherVocab);
        if(otherFile == null) return;

        try {
            // Call routine
            showWarningMessage("Possible loss of information when importing to Vdex");

            OtherImporter otherImporter = new OtherImporter(otherFile);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Import Other Vocabulary as Vdex", "Error importing", ex);
        }

    }

    /**
     * Dialogue routine to allow user to export a vdex vocabulary as Zthes
     */
    public static void exportZthesVocabulary(){

        // Ask for file
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "XML Files");
        File vdexFile = XMLUtils.askFileNameOpenUsingDirName("Open XML Vdex Vocabulary file", filter, folderMDVdexVocab);
        if(vdexFile == null) return;

        try {
            // Call routine
            showWarningMessage("Possible loss of information when exporting Vdex to Zthes");

			openFile2(vdexFile);

            ZthesExporter zthesExporter = new ZthesExporter(vdexFile);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Export Vdex Vocabulary as Zthes", "Error exporting", ex);
        }

    }

    // =========================================================================
    // ============================ DIALOG/WINDOW HANDLING =====================
    // =========================================================================

    /**
     * New Metadata Standalone Editor with New blank Document
     */
    public static MetadataEditorFrame launchNewMetadataEditor() {
        MetadataEditorFrame mdEditorFrame = null;

        try {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

            mdEditorFrame = new MetadataEditorFrame();
            mdEditorFrame.newDocument(MetadataSchemaController.getDefaultSchemaController());

            launchEditor(mdEditorFrame);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            if(mdEditorFrame != null) mdEditorFrame.dispose();
            ErrorDialogBox.showWarning("Error in New Metadata", "Could not create Metadata file", ex);
            mdEditorFrame = null;
        }
        //finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
            return mdEditorFrame;
        //}
    }

	/**
	* Tina Manoharan
	* 2003-08-12
 	*/

    /**
     * New Vocabulary Standalone Editor with New blank Document
     */
    public static VocabularyEditorFrame launchNewVocabularyEditor() {
        VocabularyEditorFrame vxEditorFrame = null;

        try {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

            vxEditorFrame = new VocabularyEditorFrame();
            vxEditorFrame.newDocument(VocabularySchemaController.getDefaultSchemaController());

            launchEditor(vxEditorFrame);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            if(vxEditorFrame != null) vxEditorFrame.dispose();
            ErrorDialogBox.showWarning("Error in New Vocabulary", "Could not create Vocabulary file", ex);
            vxEditorFrame = null;
        }
        //finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
            return vxEditorFrame;
        //}
    }


    /**
     * New Content Package and Editor
     */
    public static ContentPackageEditor launchNewContentPackageEditor() {
        ContentPackageEditor cpEditorFrame = null;

        try {
            // Ask for New CP Folder
            File cpFolder = ReloadDocumentHandler.askNewContentPackageFolder("Select Folder for New Content Package");
            // User cancelled or Project Folder not valid
            if(cpFolder == null) return null;

            // Load CP Editor
            ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

            cpEditorFrame = new ContentPackageEditor();
            cpEditorFrame.newContentPackage(cpFolder);

            launchEditor(cpEditorFrame);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            if(cpEditorFrame != null) cpEditorFrame.dispose();
            ErrorDialogBox.showWarning("Error in New Content Package", "Could not create Content Package", ex);
            cpEditorFrame = null;
        }
        //finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
            return cpEditorFrame;
        //}
    }

    /**
     * Brand new SCORM Package and Editor
     */
    public static SCORM1_2PackageEditor launchNewSCORM1_2PackageEditor() {
        SCORM1_2PackageEditor scormEditorFrame = null;

        try {
            // Ask for New CP Folder
            File cpFolder = ReloadDocumentHandler.askNewContentPackageFolder("Select Folder for new SCORM Package");
            // User cancelled or Project Folder not valid
            if(cpFolder == null) return null;

            // Load SCORM Editor
            ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

            scormEditorFrame = new SCORM1_2PackageEditor();
            scormEditorFrame.newContentPackage(cpFolder);

            launchEditor(scormEditorFrame);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            if(scormEditorFrame != null) scormEditorFrame.dispose();
            ErrorDialogBox.showWarning("Error in New SCORM Package", "Could not create SCORM Package", ex);
            scormEditorFrame = null;
        }
        //finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
            return scormEditorFrame;
        //}
    }

    /**
     * Launch and Show an Editor Window later in the Event queue
     */
    protected static void launchEditor(final ReloadInternalFrame editorWindow) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                ReloadEditor.getInstance().addInternalFrame(editorWindow);
                editorWindow.show();
            }
        });
    }

    /**
     * Ask for a file to open - xml or archive files
     */
    public static File askManifestFileOpen() {
        // Make a Filter
        javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return true;
                return file.getName().equalsIgnoreCase(MANIFEST_NAME)
                || file.getName().toLowerCase().endsWith(".zip");
            }
            public String getDescription() {
                return "XML or Archive Files";
            }
        };
        return XMLUtils.askFileNameOpen("Open XML or Archive file", filter);
    }

    /**
     * Ask to Unzip a Content Package Archive File
     * @return the Unzipped imsmanifest.xml file or null
     */
    public static File unzipContentPackage(File zipFile) throws ReloadException {
        File manifestFile;
        ReloadProgressMonitor progressMonitor = null;

        try {
            // First check that archive contains a manifest file
            if(!containsManifest(zipFile)) {
                // no manifest found in archive
                JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                          "Manifest file " + ReloadDocumentHandler.MANIFEST_NAME +
                          " cannot be located in archive.", "Archive error", JOptionPane.WARNING_MESSAGE);
                return null;
            }

            // Ask for new project folder
            File theFolder = askNewContentPackageFolder("Select Folder to Unzip to");
            // If the folder already contains the manifest or there was some other problem then return
            if(theFolder == null) return null;

            // OK up to now? ...then unzip to project folder...

            progressMonitor = new ReloadProgressMonitor(ReloadEditor.getInstance(), "Unzipping Content Package",
                "Unzipping...", "Searching...", true);

            boolean result = ZipUtils.unpackZip(zipFile, theFolder, progressMonitor);

            // Pressed Cancel
            if(!result) return null;

            // Get the unzipped manifest
            manifestFile = new File(theFolder, ReloadDocumentHandler.MANIFEST_NAME);

            // Make sure it is there in case something went wrong...
            if(!manifestFile.exists()) return null;
        }
        catch(Exception ex) {
            throw new ReloadException(ex);
        }
        finally {
            if(progressMonitor != null) progressMonitor.close();
        }

        return manifestFile;
    }

    /**
     * Ask for a New CP Project Folder
     * @return the Folder or null if cancelled or folder not valid
     */
    public static File askNewContentPackageFolder(String title) {
        // Ask for the File Name
        ReloadFolderChooser chooser = new ReloadFolderChooser();
        chooser.setDialogTitle(title);
        int returnVal = chooser.showOpenDialog(ReloadEditor.getInstance());

        // User Cancelled
        if(returnVal != ReloadFolderChooser.APPROVE_OPTION) return null;

        // Get the chosen File
        File file = chooser.getSelectedFileAndStore();

        // Sanity check
        if(!file.exists() || !file.isDirectory()) {
            JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                "Invalid Content Package Folder!", "New Content Package", JOptionPane.WARNING_MESSAGE);
            return null;
        }

        // If there is already an imsmanifest.xml file warn the user
        File imsManifest = new File(file, "imsmanifest.xml");
        if(imsManifest.exists()) {
            JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                "Folder already contains imsmanifest.xml", "New Content Package", JOptionPane.WARNING_MESSAGE);
            return null;
        }

        return file;
    }

    /**
     * Looks for imsmanifest.xml in a zip archive
     * @returns either true or false
     */
    public static boolean containsManifest(File zipFile) throws ReloadException {
        String manifestFile = ZipUtils.extractZipEntry(zipFile, MANIFEST_NAME);
        return manifestFile != null;
    }

    /**
     * Warn the user that we can't cope with an old version
     */
    protected static void showWarningMessage(String msg) {
        JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                                      msg,
                                      "Reload Editor",
                                      JOptionPane.WARNING_MESSAGE);
    }
}