package uk.ac.reload.metadata.vdexBrowser;

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import org.jdom.*;
import uk.ac.reload.widgets.tree.*;


/**
 * A Browser Tree that will display the hierarchical contents of an Vdex XML File.
 *
 * @author Tina Manoharan
 * @version 2003-09-11
 */
public class VdexTree
extends ReloadTree
implements TreeSelectionListener
{
	private VdexTermPanel termPanel;

	private String nodeName;

	private ArrayList nodeNames;

    /**
     * Constructor
     */
    public VdexTree() {
        super();
        addTreeSelectionListener(this);

        setCellRenderer(new ReloadTreeRenderer());

        nodeName = "";

        nodeNames = new ArrayList();
    }

    public void setTermPanel(VdexTermPanel termPanel) {

		this.termPanel = termPanel;
	}

    /**
     * A node selection has been made on the tree.  This will call setElement(element)
     * on the editor and then display the node's information values.
     * @param event The TreeSelectionEvent fired from us being a TreeSelectionListener
     */
    public void valueChanged(TreeSelectionEvent event) {

        if(event == null) return;
        TreePath selPath = event.getPath();
        if(selPath == null) return;

        // Get selected node
        VdexTreeNode selectedNode = (VdexTreeNode)selPath.getLastPathComponent();
        if(selectedNode == null) return;

		// get the current nodeName
		nodeName = (String)selectedNode.getUserObject();
        Element element = selectedNode.getElement();
        termPanel.setElement(element);

        // get all node names from the current one up to the root
        Object[] selectedNodes = selPath.getPath();

		// clear nodeNames first before adding things to it
		nodeNames.clear();

        for(int i = 0; i < selectedNodes.length; i++) {

			if(selectedNodes[i] instanceof VdexTreeNode) {

				selectedNode = (VdexTreeNode) selectedNodes[i];

				String temp = (String) selectedNode.getUserObject();

				if(temp != null && !nodeNames.contains(temp) && !temp.equals("Root")) nodeNames.add(temp);
			}
		}
    }


    public String getNodeName() {

		return nodeName;
	}

	public String[] getAllNodeNames() {

		return (String[]) nodeNames.toArray(new String[nodeNames.size()]);
	}


	public void highlightNode(String search) {


		TreeModel treeModel = this.getModel();
		VdexTreeNode rootNode = (VdexTreeNode) treeModel.getRoot();

		Enumeration enum = rootNode.depthFirstEnumeration();

		VdexTreeNode node = null;
		String name;
		int index;
		boolean found = false;


		while(enum.hasMoreElements()) {

			node = (VdexTreeNode) enum.nextElement();

			name = node.getNodeName();

			index = name.indexOf(":");

			if(index != -1)	{

				name = name.substring(0,index);

				name.trim();

				if(name.startsWith(search)) {

					found = true;
					break;
				}
			}
			else {

				if(name.startsWith(search)) {

					found = true;
					break;
				}
			}
		}

		if(found && node!=null) {

			TreePath treePath = new TreePath(node.getPath());
			this.setSelectionPath(treePath);
		}

	}

}


/*

	public void highlightNode(String nodeName) {

		// 1: find the appropriate node by searching for begins with a (a : b) in the tree

		TreeModel treeModel = this.getModel();

		VdexTreeNode foundNode = this.findNode((VdexTreeNode) treeModel.getRoot(), nodeName);

		if(foundNode != null) {

			// 2: highlight the node if found
		}
	}

	public VdexTreeNode findNode(VdexTreeNode node, String search) {

		for(int i = 0; i < node.getChildCount(); i++) {

			VdexTreeNode next = (VdexTreeNode) node.getChildAt(i);

			String nodeName = next.getNodeName();

			if(nodeName.startsWith(search)) return next;

			if(!next.isLeaf()) findNode(next,search);
		}

		return null;
	}

*/