
package uk.ac.reload.metadata.vdexBrowser;

import java.util.*;
import org.jdom.*;

import uk.ac.reload.widgets.tree.*;
import uk.ac.reload.vocabulary.*;
import uk.ac.reload.schema.*;


/**
 * A Browser TreeModel that will display the hierarchical contents of a Vdex XML File.
 *
 * @author Tina Manoharan
 * @version 2003-09-11
 */
public class VdexTreeModel
extends ReloadTreeModel
{

	private VdexVocabNode vdexVocabNode;

	private	String VOCAB_NAME = "vocabName";
	private	String VOCAB_ID = "vocabIdentifier";
	private	String VOCAB_MD = "metadata";
	private	String LANGSTRING = "langstring";
	private	String LANGUAGE = "language";


	private	String TERM = "term";
	private	String TERM_ID = "termIdentifier";
	private	String TERM_CAPTION = "caption";

	private String tempTermID, tempTermCaption, tempTermVal;

	/**
     * Default Constructor
     */
    public VdexTreeModel(Vocabulary vocabulary) {

		vdexVocabNode = new VdexVocabNode();

		root = new VdexTreeNode("Root", vocabulary.getRootElement());

        buildChildren((VdexTreeNode)root);
    }


    /**
     * Build up child nodes
     * @param node The Parent Node
     */
    public void buildChildren(VdexTreeNode node) {
        Iterator children = node.getElement().getChildren().iterator();

        while(children.hasNext()) {
            Element child = (Element)children.next();

            if(doShowNode(child)) {

				if(child.getName().equals(TERM)) {

	                VdexTreeNode newNode = new VdexTreeNode(child.getName(), child);
	                node.add(newNode);
	                buildChildren(newNode);
				}
				else if(child.getName().equals(TERM_ID)) {

					tempTermID = child.getText();
				}
				else if(child.getName().equals(TERM_CAPTION)) {

					tempTermCaption = child.getChildTextTrim(LANGSTRING, child.getNamespace());

					if(tempTermID != null && tempTermCaption != null) {

						tempTermVal = tempTermID + " : " + tempTermCaption;
					}
					else if(tempTermID != null) {

						tempTermVal = tempTermID + " : " + "-";
					}
					else if(tempTermCaption != null) {

						tempTermVal = "-" + " : " + tempTermCaption;
					}
					else tempTermVal = "- : -";

	                //VdexTreeNode newNode = new VdexTreeNode(tempTermVal, child.getParent());
	                //node.add(newNode);

	                node.setNodeName(tempTermVal);
				}
            }
        }
    }


    /**
     * @return Whether to display an Element on the Tree
     */
    protected boolean doShowNode(Element element) {

		String name = element.getName();

		if(name.equals(TERM) || name.equals(TERM_ID) || name.equals(TERM_CAPTION)) {

			return true;
		}
		else if(name.equals(VOCAB_NAME) || name.equals(VOCAB_ID) || (name.equals(VOCAB_MD) && element.getParent().isRootElement())) {

			buildVdexVocabNode(element);

			return false;
		}

        return false;
    }


    /**
     * Build the Vdex Vocab Node
     * @param element
     */
    public void buildVdexVocabNode(Element element) {

		String name = element.getName();

		if(name.equals(VOCAB_NAME)) {

			vdexVocabNode.setVocabName(element.getChildTextTrim(LANGSTRING, element.getNamespace()));

			Element attribElem = element.getChild(LANGSTRING,element.getNamespace());

			if(attribElem != null) {

				vdexVocabNode.setVocabLang(attribElem.getAttributeValue(LANGUAGE));
			}
		}
		else if(name.equals(VOCAB_ID)) {

			vdexVocabNode.setVocabID(element.getText());
		}
		else if(name.equals(VOCAB_MD) && element.getParent().isRootElement()) {

			//vdexVocabNode.setVocabMD(element.getText());


			Iterator contentList = element.getContent().iterator();

			while(contentList.hasNext()) {

				Object obj = contentList.next();

				if(obj instanceof Element) {

					Element objElem = (Element) obj;

					// System.out.println("Element obj = " + objElem.getName());
				}
				else if(obj instanceof Text) {

					Text objText = (Text) obj;

					// System.out.println("Text obj = " + objText.getTextTrim());
				}
				else if(obj instanceof Comment) {

					Comment objComment = (Comment) obj;

					// System.out.println("Comment obj = " + objComment.getText());

					// Assumption Metadata provided as Comment

					vdexVocabNode.setVocabMD(objComment.getText());
				}
				else if(obj instanceof CDATA) {

					CDATA objCData = (CDATA) obj;

					// System.out.println("CDATA obj = " + objCData.getText());
				}
			}
		}
	}


    /**
     * Get the Vdex Vocab Node
     */
	public VdexVocabNode getVdexVocabNode() {

		return vdexVocabNode;
	}


}