/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.schema.viewer;

import javax.swing.event.*;
import javax.swing.tree.*;

import uk.ac.reload.schema.*;
import uk.ac.reload.widgets.tree.*;


/**
 * A Viewer Tree that will display the hierarchical contents of an XML Schema.
 * The TestTreeModel class takes care of taking the Schema and creating tree nodes.
 * A NodeViewer TextPane will display the information for a node when it is selected.
 * This is for testing purposes only.
 *
 * @author Phillip Beauvoir
 * @version 2003-04-02
 */
public class SchemaTree
extends ReloadTree
implements TreeSelectionListener
{
    /**
     * The Node viewer pane
     */
    protected NodeViewer nodeViewer;

    /**
     * Constructor
     * @param nodeViewer The NodeViewer TextPane that will display the information
     * for a selected node.
     */
    public SchemaTree(NodeViewer nodeViewer) {
        super();
        this.nodeViewer = nodeViewer;
        addTreeSelectionListener(this);

        setCellRenderer(new ReloadTreeRenderer());
    }

    /**
     * A node selection has been made on the tree.  This will call setElement(element)
     * on the NodeViewer and then display the node's information values.
     * @param event The TreeSelectionEvent fired from us being a TreeSelectionListener
     */
    public void valueChanged(TreeSelectionEvent event) {
        if(event == null) return;
        TreePath selPath = event.getPath();
        if(selPath == null) return;

        // Get selected node
        SchemaTreeNode selectedNode = (SchemaTreeNode)selPath.getLastPathComponent();
        if(selectedNode == null) return;

        ReloadSchemaElement element = (ReloadSchemaElement)selectedNode.getUserObject();
        nodeViewer.setElement(element);
    }
}