/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.util.htmlparser;

import java.io.*;
import java.util.*;
import uk.ac.reload.*;

 /**
  * Class to read in a css file and find any local links
  * Tested with CSS files containing links that were created with
  *
  * 1. Macromedia Dreamweaver MX
  * 2. TopStyle Pro 3.10
  *
  * @author Paul Sharples
  * @version 2003-04-30
  */
public class CSSParser{

     /**
      *  A vector to hold any links that are found.
      */
    Vector _linksFound = new Vector();

    /**
     * Constructor - starts the parsing.
     * @param thefile
     */
    public CSSParser(File thefile) {
        startit(thefile);
    }

    /**
     * Accessor method to return any links that were found
     * @return
     */
    public String[] getLinks() {
        String[] resources = new String[_linksFound.size()];
        _linksFound.copyInto(resources);
        return resources;
    }


    /**
     * Method to do the actual parsing.  Actually its just a simple file read
     * which checks for the existence of a substring "url(" which would denote
     * a link of some sort.
     * @param thefile
     */
    protected void startit(File thefile) {
        String line, aLink;
        int startcount, endcount;
        try {
            LineNumberReader inputFile = new LineNumberReader(new FileReader(thefile));
            while((line = inputFile.readLine()) != null) {

                startcount = line.indexOf("url(");
                if(startcount != -1) {
                    endcount = line.indexOf(");");
                    aLink = line.substring(startcount + 4, endcount);
                    if(!(aLink.toLowerCase().startsWith("http:") || aLink.toLowerCase().startsWith("https:")
                         || aLink.toLowerCase().startsWith("mailto:"))) {
                        _linksFound.add(aLink);
                    }
                }
            }
            inputFile.close();
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Import Resources", "Error importing in CSS file", ex);
        }
    }
}