/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.vocabulary.importexport;

import java.io.*;

import org.jdom.*;

import javax.xml.transform.*;
import javax.xml.transform.stream.*;

//import com.icl.saxon.*;
//import com.icl.saxon.jdom.*;
//import com.icl.saxon.expr.*;
//import com.icl.saxon.om.*;

import org.jdom.transform.*;

import uk.ac.reload.*;
import uk.ac.reload.dom.ReloadDocumentHandler;
import uk.ac.reload.util.XMLUtils;
import uk.ac.reload.vocabulary.*;


public class ZthesImporter {

	public ZthesImporter(File zthesFile) {

		try {

			// create source document

			Document zthesDoc = XMLUtils.readXMLFile(zthesFile);

	    	// Now do a transformation

	    	File stylesheetFile = ReloadDocumentHandler.ZTHES_TO_VDEX_STYLESHEET;

			Transformer transformer = TransformerFactory.newInstance().newTransformer(new StreamSource(stylesheetFile));


			// create result by performing transformation using stylesheet

			JDOMSource source = new JDOMSource(zthesDoc);

			JDOMResult result = new JDOMResult();

			transformer.transform(source, result);

			Document vdexDoc = result.getDocument();


			// save output file

	    	File vdexOutputFile = new File(ReloadDocumentHandler.folderMDVdexVocab, zthesFile.getName());

	    	XMLUtils.write2XMLFile(vdexDoc, vdexOutputFile);


	    	// open output file in editor

	    	if(vdexOutputFile.canRead()) ReloadDocumentHandler.openFile2(vdexOutputFile);

		}
        catch(ReloadException rex) {
            if(ReloadEditor.DEBUG) rex.printStackTrace();
            ErrorDialogBox.showWarning("Import Zthes Vocabulary as Vdex", "Error importing", rex);
        }
        catch(TransformerConfigurationException tex) {
            if(ReloadEditor.DEBUG) tex.printStackTrace();
        }
        catch(TransformerException tex2) {
            if(ReloadEditor.DEBUG) tex2.printStackTrace();
        }

	}
}


	/*    Using Saxon XSLT Processor

	public ZthesImporter(File zthesFile) {

		try {

			Document zthesDoc = XMLUtils.readXMLFile(zthesFile);

			// Give it a Saxon wrapper
	    	DocumentWrapper saxonZthesDocWrap = new DocumentWrapper(zthesDoc, zthesFile.getAbsolutePath());
	    	NamePool pool = NamePool.getDefaultNamePool();
	    	saxonZthesDocWrap.setNamePool(pool);

	    	// Now do a transformation

	    	System.setProperty("javax.xml.transform.TransformerFactory", "com.icl.saxon.TransformerFactoryImpl");
	    	TransformerFactory tfactory = TransformerFactory.newInstance();

	    	String stylesheetFileName = System.getProperty("user.dir") + ReloadUIManager.ZTHES_TO_VDEX_STYLESHEET;

	    	Templates templates = tfactory.newTemplates(new StreamSource(stylesheetFileName));
	    	Transformer transformer = templates.newTransformer();

	    	File vdexOutputFile = new File(ReloadDocumentHandler.folderMDVdexVocab, zthesFile.getName());

	    	transformer.transform(saxonZthesDocWrap, new StreamResult(vdexOutputFile));

	    	if(vdexOutputFile.canRead()) ReloadDocumentHandler.openFile2(vdexOutputFile);
		}
        catch(ReloadException rex) {
            if(ReloadEditor.DEBUG) rex.printStackTrace();
            ErrorDialogBox.showWarning("Import Zthes Vocabulary as Vdex", "Error importing", rex);
        }
        catch(TransformerConfigurationException tex) {
            if(ReloadEditor.DEBUG) tex.printStackTrace();
        }
        catch(TransformerException tex2) {
            if(ReloadEditor.DEBUG) tex2.printStackTrace();
        }
	}
	*/

