/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.widgets;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import org.jdom.*;

import uk.ac.reload.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.util.*;
import uk.ac.reload.metadata.vdexBrowser.*;

/**
 * A Combination of JTextField and JButton for selecting files
 *
 * @author Tina Manoharan
 * @version 2003-09-10
 */
public class ReloadButtonTextField
extends JPanel
implements ReloadWidget
{
    /**
     * The maximum number of characters allowed
     */
    protected int _maxLength;

    /**
     * A Textfield to display the value
     */
    protected JTextField _textField;

    /**
     * A button for browing Vdex vocabulary values
     */
    protected JButton _button;


    /**
     * The text to display on button
     */
    protected String _text;

    /**
     * The button type
     */
    protected int _type;

    /**
     * The Vdex Source Path
     */
    protected String _vdexSourcePath;


    /**
     * The Vdex Source Value
     */
    protected String _vdexSourceValue;


    /**
     * Type of button - BROWSE_TYPE
     */
    public static int VDEX_BROWSE_TYPE = 1;

    /**
     * The Reload Document
     */
    protected ReloadDocument _reloadDocument;

    protected ReloadSchemaElement _schemaElement;



    /**
     * Constructor
     */
    public ReloadButtonTextField(ReloadDocument reloadDocument, int type, String text) {
        super(new BorderLayout());

		_reloadDocument = reloadDocument;
        _type = type;
        _text = text;

        // Add text field
        _textField = new JTextField();
        _textField.setFont(ReloadUIManager.plainFont11);
        add(_textField, BorderLayout.CENTER);

        // Add Button
        _button = new JButton(_text, ReloadUIManager.getIcon(ReloadUIManager.ICON_VXBROWSE));
        add(_button, BorderLayout.EAST);
        setupAction();
    }

    /**
     * Set up the button action
     * @param button
     */
    protected void setupAction() {
        _button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

				if(_type == VDEX_BROWSE_TYPE) {

		        	Thread thread = new Thread() {
		        	    public void run() {
		        	        try {

		        	            // ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

		        	            // first get the Vdex Source Value
		        	            handleVdexBrowse();

		        	            // New Vdex Browser Window
		        	            final MetadataVdexBrowser mdVdexBrowser = new MetadataVdexBrowser(ReloadButtonTextField.this, _vdexSourceValue, "Browse Vdex Vocabulary");

		        	            // Show the Window later in the Event queue
		        	            SwingUtilities.invokeLater(new Runnable() {
		        	                public void run() {
		        	                    //ReloadEditor.getInstance().addInternalFrame(mdVdexBrowser);

		        	                    mdVdexBrowser.show();
		        	                }
		        	            });

		        	        }
		        	        catch(Exception ex) {
		        	            ex.printStackTrace();
		        	        }
		        	        //finally {
		        	            //ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
		        	        //}
		        	    }
		        	};

		        	thread.start();
				}

				//ReloadEditor.getInstance().setCursor(ReloadUIManager.TEXT_CURSOR);
			}
		});
    }


    /**
     * Get the Text Field
     * @return
     */
    public JTextField getTextField() {
        return _textField;
    }

    /**
     * Set the Value
     * @param value
     */
    public void setTextValue(String value) {
        _textField.setText(value);
    }

    /**
     * Insert some text
     */
    public void insertTextValue(String text) {

    }

    /**
     * Get the Value
     * @param value
     */
    public String getTextValue() {
        return _textField.getText();
    }

    /**
     * Set the maximum allowed of digits
     * @param maxLength
     */
    public void setMaxLength(int maxLength) {
        _maxLength = maxLength;
    }

    /**
     * @return whether this widget only needs a single line to display itself
     */
    public boolean isSingleLine() {
        return true;
    }

    /**
     * Add a DocumentListener to the Text Box
     * @param dl
     */
    public void addDocumentListener(DocumentListener dl) {
        _textField.getDocument().addDocumentListener(dl);
    }

    /**
     * Remove a DocumentListener from the Text Box
     * @param dl
     */
    public void removeDocumentListener(DocumentListener dl) {
        _textField.getDocument().removeDocumentListener(dl);
    }

    /**
     * Set the components enabled/disabled
     * @param enabled
     */
    public void setEnabled(boolean enabled) {
        _textField.setEnabled(enabled);
        _button.setEnabled(enabled);
    }


    /**
     * Set the columns allowed for the textfield
     * @param maxLength
     */
    public void setColumns(int cols) {
        _textField.setColumns(cols);
    }

    /**
     * Set the Vdex Source Path
     * @param vdexSourcePath
     */
    public void setVdexSourcePath(String vdexSourcePath) {

		_vdexSourcePath = vdexSourcePath;
	}


    /**
     * Get the Vdex Source Path
     * @param vdexSourcePath
     */
    public String getVdexSourcePath() {

		return _vdexSourcePath;
	}

    /**
     * Set the Vdex Source Value
     * @param vdexSourceValue
     */
    public void setVdexSourceValue(String vdexSourceValue) {

		_vdexSourceValue = vdexSourceValue;
	}

    /**
     * Get the Vdex Source Value
     * @param vdexSourceValue
     */
    public String getVdexSourceValue() {

		return _vdexSourceValue;
	}

	public void setSchemaElement(ReloadSchemaElement schemaElement) {

		_schemaElement = schemaElement;
	}

	public ReloadSchemaElement getSchemaElement() {

		return _schemaElement;
	}

    /**
     * Handle Vdex Browsing
     */
    public void handleVdexBrowse() {

		Element element = null;

		if(_vdexSourcePath != null) {

			DomPath domPath = new DomPath(_vdexSourcePath);

			try {

				element = _reloadDocument.getElement(domPath);

				if(element.getText() != null) this.setVdexSourceValue(element.getText());

			}
			catch(NullPointerException ne) {

				showWarningMessage("Choose a Vdex source file");
			}

		}

	}

    /**
     * Warn the user that a source vdex file was not choosen
     */
    protected void showWarningMessage(String msg) {
        JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                                      msg,
                                      "Reload Editor",
                                      JOptionPane.WARNING_MESSAGE);
    }

    /**
     * Set Vdex Values selected on the Browser to ReloadDocument
     */
	public void setVdexValuesToDocument(String[] vals) {

		StringTokenizer st;

		String val1 = null, val2 = null;

		Element element, parentElement = null, newParentElement = null;

		int index;

		ReloadSchemaElement parentSchemaElement = null, newSchemaElement = null, secondSchemaElement = null;


		for(int i = 0; i < vals.length; i++) {

			val1 = null; val2 = null;

			if(vals[i].indexOf(':') != -1) {

				st = new StringTokenizer(vals[i], ":");

				while(st.hasMoreTokens()) {

					val1 = st.nextToken().trim();

					val2 = st.nextToken().trim();
				}
			}
			else {

				val1 = vals[i];
			}

			if(i == 0) {

				if(val1 != null) {

					element = _reloadDocument.getElement(_schemaElement);


					if(element != null) {

						element.setText(val1);

						_reloadDocument.changedElement(this, element);
					}
					else {

						element = _reloadDocument.addElementUnique(this, _schemaElement, true);

						if(element != null) {

							element.setText(val1);

							_reloadDocument.changedElement(this, element);

						}
					}

					// setting the parentElement
					if(_schemaElement.getName().equals("langstring")) {

						parentSchemaElement = (ReloadSchemaElement) _schemaElement.getParent().getParent();

						parentElement = element.getParent().getParent();
					}
					else {

						parentSchemaElement = (ReloadSchemaElement) _schemaElement.getParent();

						parentElement = element.getParent();
					}

				}

				// Assumption: 2nd element if exists is at the same level as the first

				if(val2 != null) {

					index = parentSchemaElement.indexofChild(_schemaElement);

					index++;

					secondSchemaElement = parentSchemaElement.getChildAt(index);

					if(secondSchemaElement != null) {


						if(secondSchemaElement.hasChild("langstring")) {

							element = _reloadDocument.getElement(secondSchemaElement.getChild("langstring"));
						}
						else element = _reloadDocument.getElement(secondSchemaElement);

						if(element != null) {

							element.setText(val2);

							_reloadDocument.changedElement(this, element);
						}
						else {

							element = _reloadDocument.addElementUnique(this, secondSchemaElement, true);

							if(secondSchemaElement.hasChild("langstring")) {

								element = element.getChild("langstring", element.getNamespace());
							}

							if(element != null) {

								element.setText(val2);

								_reloadDocument.changedElement(this, element);

							}
						}
					}
				}
			}
			else { // if (i > 0)

				// Note: All elements (e.g. Taxons) are in a hierarchy, this is according to Reload IMS Metadata Schema

				if(val1 != null && parentElement != null && parentSchemaElement != null) {

					if(parentSchemaElement.hasChild(parentSchemaElement.getName())) {

						// hierarchy
						// TaxonPath -> taxon -> taxon

						newSchemaElement = parentSchemaElement.getChild(parentSchemaElement.getName(), parentSchemaElement.getNamespace());
					}
					else {

						// same level
						// TaxonPath -> taxon
						// 			 -> taxon

						newSchemaElement = parentSchemaElement;

						parentElement = parentElement.getParent();
					}

					if(_schemaElement.getName().equals("langstring")) {

						newParentElement = _reloadDocument.addElementBySchemaUndoable(this, parentElement, newSchemaElement, true);


						element = _reloadDocument.addElementBySchemaUndoable(this, newParentElement, (ReloadSchemaElement)_schemaElement.getParent(), true);


						element = element.getChild("langstring", element.getNamespace());
					}
					else {

						newParentElement = _reloadDocument.addElementBySchemaUndoable(this, parentElement, newSchemaElement, true);


						element = _reloadDocument.addElementBySchemaUndoable(this, newParentElement, _schemaElement, true);

					}

					if(element != null) {

						element.setText(val1);

						_reloadDocument.changedElement(this, element);
					}

				}


				if(val2 != null && newParentElement != null && parentSchemaElement != null && secondSchemaElement != null) {

					if(secondSchemaElement != null) {

						if(secondSchemaElement.hasChild("langstring")) {

							element = _reloadDocument.addElementBySchemaUndoable(this, newParentElement, secondSchemaElement, true);


							element = element.getChild("langstring", element.getNamespace());
						}
						else {

							element = _reloadDocument.addElementBySchemaUndoable(this, newParentElement, secondSchemaElement, true);
						}

						if(element != null) {

							element.setText(val2);

							_reloadDocument.changedElement(this, element);
						}
					}
				}


				// setting parentElement
				parentElement = newParentElement;

				// setting parentSchemaElement
				parentSchemaElement = newSchemaElement;
			}
		}
	}


}
