/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.widgets;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

import uk.ac.reload.*;
import uk.ac.reload.schema.*;

/**
 * A Combination of JTextField and JList
 *
 * @author Phillip Beauvoir
 * @version 2003-07-30
 */
public class ReloadListBox
extends JPanel
implements ReloadWidget, ListSelectionListener, ReloadVocabularyListener
{
    /**
     * The maximum number of characters allowed
     */
    protected int _maxLength;

    /**
     * A Textfield to display the value
     */
    protected JTextField textField;

    /**
     * A list box to display the list
     */
    protected JList listBox;

    protected ReloadVocabularyList _rvList;

    /**
     * Constructor
     */
    public ReloadListBox() {
        super(new BorderLayout());

        // Add text field
        textField = new JTextField();
        textField.setFont(ReloadUIManager.plainFont11);
        add(textField, BorderLayout.NORTH);

        // Add List
        listBox = new JList();
        listBox.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        add(new JScrollPane(listBox), BorderLayout.CENTER);

        // Listen to List Selections
        listBox.addListSelectionListener(this);
    }

    /**
     * Get the Text Field
     * @return
     */
    public JTextField getTextField() {
        return textField;
    }

    /**
     * Get the List Box
     * @return
     */
    public JList getListBox() {
        return listBox;
    }

    /**
     * Set the Lists Data
     * @param values
     */
    public void setListData(Object[] values) {
        listBox.setListData(values);
    }

    /**
     * Set the VocabularyList
     * @param values
     */
    public void setList(ReloadVocabularyList rvList) {

        _rvList = rvList;

        // We listen to VocabularyList changes
        _rvList.addReloadVocabularyListener(this);
    }

    public void destroy() {
        _rvList.removeReloadVocabularyListener(this);
    }

    /**
     * Set the Value
     * @param value
     */
    public void setTextValue(String value) {
        listBox.setSelectedValue(value, true);
        textField.setText(value);
        //textField.requestFocus();
    }

    /**
     * Insert some text
     */
    public void insertTextValue(String text) {

    }

    /**
     * Get the Value
     * @param value
     */
    public String getTextValue() {
        return textField.getText();
    }

    /**
     * Set the maximum allowed of digits
     * @param maxLength
     */
    public void setMaxLength(int maxLength) {
        _maxLength = maxLength;
    }

    /**
     * @return whether this widget only needs a single line to display itself
     */
    public boolean isSingleLine() {
        return false;
    }

    /**
     * Add a DocumentListener to the Text Box
     * @param dl
     */
    public void addDocumentListener(DocumentListener dl) {
        textField.getDocument().addDocumentListener(dl);
    }

    /**
     * Remove a DocumentListener from the Text Box
     * @param dl
     */
    public void removeDocumentListener(DocumentListener dl) {
        textField.getDocument().removeDocumentListener(dl);
    }

    /**
     * Set the components enabled/disabled
     * @param enabled
     */
    public void setEnabled(boolean enabled) {
        textField.setEnabled(enabled);
        listBox.setEnabled(enabled);
    }

    /**
     * ListSelection Listener for the Vocabulary List and update the Text Field
     */
    public void valueChanged(ListSelectionEvent e) {
        if(e.getSource() == listBox) {
            Object object = listBox.getSelectedValue();
            if(object != null) textField.setText(object.toString());
        }
    }

// ===================================================================================
// ===================== These events are received FROM the ReloadVocabularyList =====
// ===================================================================================

    public void vocabularyListChanged(ReloadVocabularyListenerEvent event) {

		// list changed update combobox

		listBox.setListData(event.getList());
    }

}