/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.widgets.tree;

import javax.swing.*;

import org.jdom.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.schema.*;



/**
 * An implementation of DefaultMutableTreeNode for Trees.  It contains:
 * A JDOM Element
 * A SchemaElement
 * A ReloadDocument
 *
 * @author Phillip Beauvoir
 * @version 2003-08-04
 */
public abstract class ReloadElementTreeNode
extends ReloadTreeNode
{
    /**
     * The associated ReloadSchemaElement
     */
    protected ReloadSchemaElement _reloadSchemaElement;

    /**
     * The Parent Document
     */
    protected ReloadDocument _reloadDocument;

    /**
     * The ReloadElementBinding
     */
    protected ReloadElementBinding _eb;


    /**
     * Constructor for Root Element
     * @param reloadDocument The ReloadDocument DOM Document
     */
    public ReloadElementTreeNode(ReloadDocument reloadDocument) {
        super(reloadDocument.getRootElement());
        _reloadSchemaElement = reloadDocument.getReloadSchemaController().getReloadSchema().getRootElement();
        _reloadDocument = reloadDocument;
    }

    /**
     * Constructor for child Element
     * @param element The JDOM Element
     */
    public ReloadElementTreeNode(Element element) {
        super(element);
    }

    /**
     * Because other classes may not be interested in tree nodes we can return
     * the important bits wrapped up in a ReloadElementBinding
     * @return ReloadElementBinding
     */
    public ReloadElementBinding createReloadElementBinding() {
        if(_eb == null) _eb = new ReloadElementBinding(getElement(), getSchemaElement(), getReloadDocument());

        return _eb;
    }

    /**
     * Get the JDOM Element
     * @return The JDOM Element
     */
    public Element getElement() {
        return (Element)getUserObject();
    }

    /**
     * Get the ReloadSchemaElement
     * @return The associated ReloadSchemaElement
     */
    public ReloadSchemaElement getSchemaElement() {
        if(_reloadSchemaElement == null && getParent() != null) {
            _reloadSchemaElement = ((ReloadElementTreeNode) getParent()).getSchemaElement();
            if(_reloadSchemaElement != null) _reloadSchemaElement = _reloadSchemaElement.getChild(getElement().getName(), getElement().getNamespace());
        }

        return _reloadSchemaElement;
    }

    /**
     * @return the Parent Document
     */
    public ReloadDocument getReloadDocument() {
        if(_reloadDocument == null && getParent() != null) _reloadDocument = ((ReloadElementTreeNode)getParent()).getReloadDocument();
        return _reloadDocument;
    }

    /**
     * @return the name of the Element
     */
    public String getName() {
        return getElement().getName();
    }

    /**
     * @return the Leaf Icon to display
     */
    public Icon getLeafIcon() {
        return getReloadDocument().getReloadSchemaController().getLeafIcon(getElement());
    }

    /**
     * @return the Open Icon to display
     */
    public Icon getOpenIcon() {
        return getReloadDocument().getReloadSchemaController().getOpenIcon(getElement());
    }

    /**
     * @return the Closed Icon to display
     */
    public Icon getClosedIcon() {
        return getReloadDocument().getReloadSchemaController().getClosedIcon(getElement());
    }

}
