/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.contentpackaging.manifestview;

import org.jdom.*;
import uk.ac.reload.contentpackaging.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.widgets.*;

/**
 * A Content Package Manifest Elemetn Editor
 *
 * @author Phillip Beauvoir
 * @version 2003-08-01
 */
public class ManifestElementEditor
extends ReloadElementEditor {

    /**
     * Whether we display a child Title value
     */
    protected boolean isTitleValue;

    /**
     * The Title Element if we are editing an Item or Organizaton
     */
    protected Element _titleElement;

    /**
     * Constructor
     */
    public ManifestElementEditor() {
        super();
    }

    /**
     * Set The Panel to display the data for the Element
     * Over-ride this so we can get the Title Element
     */
    public void setElement(ReloadElementBinding elementBinding) {
        // Remove old Listener
        if(_elementBinding != null) _elementBinding.getReloadDocument().removeReloadDocumentListener(this);

        _elementBinding = elementBinding;

        // Add Listener
        _elementBinding.getReloadDocument().addReloadDocumentListener(this);

        Element element = _elementBinding.getElement();
        String name = element.getName();

        // ORGANIZATION & ITEM elements display the child TITLE element
        if(name.equals(ContentPackage.ORGANIZATION) || name.equals(ContentPackage.ITEM)) {
            if(_elementBinding.getReloadDocument().isDocumentNamespace(element)) {
                isTitleValue = true;
                ReloadSchemaElement schemaElement = _elementBinding.getSchemaElement();
                if(schemaElement != null) schemaElement = schemaElement.getChild(ContentPackage.TITLE);
                // Set this to the Title element
                _titleElement = element.getChild(ContentPackage.TITLE, element.getNamespace());
                setupWidget(_elementBinding.getReloadDocument(), _titleElement, schemaElement);
            }
        }

        else {
            isTitleValue = false;
            setupWidget(_elementBinding.getReloadDocument(), _elementBinding.getElement(), _elementBinding.getSchemaElement());
        }
    }

    /**
     * Over-ride to do stuff
     */
    protected void setupWidget(ReloadDocument doc, Element element, ReloadSchemaElement schemaElement) {
        super.setupWidget(doc, element, schemaElement);

        if(_reloadWidget != null) {
            // SCORM location widget needs relative path offset
            if(_reloadWidget instanceof ReloadFileTextField) {
                if(schemaElement.getName().equals("location") && doc instanceof ContentPackage) {
                    ((ReloadFileTextField)_reloadWidget).setRelativePath(((ContentPackage)doc).getProjectFolder());
                }
            }
        }
    }

    /**
     * Over-ride to trap this
     */
    public void elementChanged(ReloadDocumentListenerEvent event) {
       // Don't listen to events that we have fired
       if(event.getSource() == this) return;

       // Are we changing the Title Element?
       if(isTitleValue && _titleElement != null) {
           Element changed_element = event.getElement();
           if(_titleElement == changed_element) setText(_titleElement);
       }
       // Nope
       else super.elementChanged(event);
   }

   /**
     * Update the Model
     */
    protected synchronized void fireElementChanged(String text) {
        // We actually edited the child Title Element
        if(isTitleValue) {
            Element element = _elementBinding.getElement();

            // If value is blank remove the element
            if(text.equals("")) {
                if(_titleElement != null) {
                    _elementBinding.getReloadDocument().removeElement(this, _titleElement);
                    _titleElement = null;
                }
            }

            // Else set Title value
            else {
                if(_titleElement == null) {
                    _titleElement = new Element(ContentPackage.TITLE, element.getNamespace());
                    _elementBinding.getReloadDocument().addElement(this, element, _titleElement, false);
                }

                _titleElement.setText(text);
            }

            // Tell the Parent
            _elementBinding.getReloadDocument().changedElement(this, element);
        }

        else super.fireElementChanged(text);
    }
}