/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.contentpackaging.manifestview;

import java.awt.*;
import javax.swing.*;

import uk.ac.reload.contentpackaging.*;
import uk.ac.reload.util.*;

/**
 * A Content Package Manifest Panel for the CP UI
 *
 * @author Phillip Beauvoir
 * @version 2003-07-27
 */
public class ManifestPanel
extends JPanel
{
    /**
     * The Tree
     */
    protected ManifestTree _manifestTree;

    /**
     * The info Panel
     */
    protected ManifestInfoPanel _infoPanel;

    /**
     * Splitpane
     */
    private JSplitPane _splitPane;

    /**
     * Constructor
     */
    public ManifestPanel(ContentPackageEditor cpEditor) {
        setLayout(new BorderLayout());

        // Create new Info panel
        _infoPanel = new ManifestInfoPanel();

        _manifestTree = new ManifestTree(cpEditor, _infoPanel);

        // Set up a Split Pane
        _splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        _splitPane.setOneTouchExpandable(true);
        _splitPane.setTopComponent(new JScrollPane(_manifestTree));
        _splitPane.setBottomComponent(_infoPanel);
        add(_splitPane, BorderLayout.CENTER);
    }

    /**
     * Set a new Content Package to be displayed
     * @param cp
     */
    public void setContentPackage(ContentPackage cp) {
        _manifestTree.setContentPackage(cp);
    }

    /**
     * Clean up
     */
    public void destroy() {
        if(_manifestTree != null) _manifestTree.destroy();
        if(_infoPanel != null) _infoPanel.destroy();
    }

    /**
     * Called after Panel is shown so we can init stuff
     */
    public void initView() {
        _splitPane.setDividerLocation(0.7);
        _infoPanel.initView();
    }

}