/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.menu;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * The Edit Menu
 *
 * @author Phillip Beauvoir
 * @version 2003-05-18
 */
public class Menu_Edit extends ReloadMenu {

    public MenuAction_Undo actionUndo;
    public MenuAction_Redo actionRedo;
    public MenuAction_Cut actionCut;
    public MenuAction_Copy actionCopy;
    public MenuAction_Paste actionPaste;
    public MenuAction_Delete actionDelete;
    public MenuAction_MoveUp actionMoveUp;
    public MenuAction_MoveDown actionMoveDown;


    public Menu_Edit() {
        super("Edit");
        setMnemonic('e');

        actionUndo = new MenuAction_Undo();
        actionRedo = new MenuAction_Redo();
        actionCut = new MenuAction_Cut();
        actionCopy = new MenuAction_Copy();
        actionPaste = new MenuAction_Paste();
        actionDelete = new MenuAction_Delete();
        actionMoveUp = new MenuAction_MoveUp();
        actionMoveDown = new MenuAction_MoveDown();

        int keyMask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

        // Undo
        JMenuItem item = add(actionUndo);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, keyMask));

        // Redo
        item = add(actionRedo);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, Event.SHIFT_MASK + keyMask));

        addSeparator();

        // Cut
        item = add(actionCut);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, keyMask));

        // Copy
        item = add(actionCopy);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, keyMask));

        // Paste
        item = add(actionPaste);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, keyMask));

        // Delete
        item = add(actionDelete);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));

        addSeparator();

        // Move Up
        item = add(actionMoveUp);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_UP, InputEvent.ALT_MASK));

        // Move Down
        item = add(actionMoveDown);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, InputEvent.ALT_MASK));
    }

    /**
     * Some components need the Ctrl-X, Ctrl-C, and Ctrl-V key bindings remapped
     * @param component
     */
    public void remapKeyStrokes(JComponent component) {
        int keyMask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
        component.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_X, keyMask), "CUT");
        component.getActionMap().put("CUT", actionCut);
        component.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_C, keyMask), "COPY");
        component.getActionMap().put("COPY", actionCopy);
        component.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_V, keyMask), "PASTE");
        component.getActionMap().put("PASTE", actionPaste);
    }

}