/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.metadata.formview;

import java.awt.*;
import java.awt.event.*;

import org.jdom.Element;

import uk.ac.reload.ReloadEditor;
import uk.ac.reload.metadata.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.widgets.*;


/**
 * A Combo Box widget.  Derived from JComboBox.
 * This can hold an Element or an Attribute.
 *
 * @author Phillip Beauvoir
 * @version 2003-03-12
 */
public class MetadataComboBox
extends MetadataField
implements ActionListener, ReloadVocabularyListener
{
    boolean allowNotification = true;

    protected ReloadComboBox _comboBox;

    protected ReloadVocabularyList _rvList;

    public MetadataComboBox(Metadata metadata, ReloadSchemaElement schemaElement, ReloadVocabularyList rvList) {
        super(metadata, schemaElement);

        _comboBox = new ReloadComboBox(true);
        _comboBox.setBorder(null);

		_rvList = rvList;
        _comboBox.setItems(_rvList.getList());

        _element = metadata.getElement(_schemaElement);
        if(_element != null) setValue(_element.getText());
        else setValue("");

        // We listen to Text Changes
        _comboBox.addActionListener(this);

        // We listen to Element changes
        metadata.addReloadDocumentListener(this);

        // We listen to VocabularyList changes
        _rvList.addReloadVocabularyListener(this);
    }


    public void destroy() {
        _reloadDocument.removeReloadDocumentListener(this);
        _comboBox.removeActionListener(this);

        _rvList.removeReloadVocabularyListener(this);
    }

    /**
     * Get the value of this Widget
     * @return The value contained in the Widget
     */
    public String getValue() {
        return (String)_comboBox.getSelectedItem();
    }

    /**
     * Set the value of the Widget
     * @param value The value
     */
    public void setValue(String value) {
        allowNotification = false;
        _comboBox.setSelectedItem(value);
        allowNotification = true;
    }

    public Component getComponent() {
        return _comboBox;
    }

    public void actionPerformed(ActionEvent event) {
        if(allowNotification) fireElementChanged();
    }


// ===================================================================================
// ===================== These events are received FROM the ReloadVocabularyList =====
// ===================================================================================

    public void vocabularyListChanged(ReloadVocabularyListenerEvent event) {

		// list changed update combobox

		_comboBox.setItems(event.getList());

		try {
			_comboBox.setSelectedIndex(-1);
		}
		catch(IllegalArgumentException ex) {
			if(ReloadEditor.DEBUG) System.out.println("ComboBox selection index out of range");
		}
    }

}