/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.metadata.formview;

import java.util.*;

import org.jdom.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.widgets.*;

/**
 * The Reload MD Form Widget
 *
 * @author Phillip Beauvoir
 * @version 2003-08-01
 */
public abstract class MetadataField
extends ReloadFormField
{
    /**
     * Constructor
     */
    public MetadataField(ReloadDocument doc, ReloadSchemaElement schemaElement) {
        super(doc, null, schemaElement);
    }

    /**
     * Tell the ReloadDocument that we changed
     */
    protected synchronized void fireElementChanged() {
        String text = getValue();

        // If we are bound to an Element
        if(_element != null) {
            _element.setText(text);
            _reloadDocument.changedElement(this, _element);
        }

        // Create new Element if we don't have one
        else {
            _element = _reloadDocument.addElementUnique(this, _schemaElement, true);
            if(_element != null) {

				_element.setText(text);

				/**
				* Tina Manoharan
				* 2003-10-28
				*/
				// added this one so that the 'source' element will update the vocabulary list for the 'value' element
				_reloadDocument.changedElement(this, _element);
			}
        }
    }


// =============================================================================
// ===================== These events are received FROM the ReloadDocument =====
// =============================================================================

    public void elementAdded(ReloadDocumentListenerEvent event) {
        // Don't listen to events that we have fired
        if(event.getSource() == this) return;

        addElement(event.getElement());
    }

    protected void addElement(Element element) {
        // Is this us?
        if(element == _reloadDocument.getElement(_schemaElement)) {
            _element = element;
            setValue(_element.getText());
        }
        // Check children
        else {
            Iterator it = element.getChildren().iterator();
            while(it.hasNext()) {
                Element child = (Element) it.next();
                addElement(child);
            }
        }
    }

    public void elementChanged(ReloadDocumentListenerEvent event) {
        // Don't listen to events that we have fired
        if(event.getSource() == this) return;

        // We changed!
        Element element = event.getElement();
        if(element == _element) {

			setValue(_element.getText());
		}
    }

    public void elementRemoved(ReloadDocumentListenerEvent event) {
        // Don't listen to events that we have fired
        if(event.getSource() == this) return;

        Element element = event.getElement();
        if(_element != null) {
            // It was an Element like us that was removed!
            if(element == _element || _element.isAncestor(element)) {
                // Is there another one like us?
                _element = _reloadDocument.getElement(_schemaElement);
                // No
                if(_element == null) {

					setValue("");
				}
                // Yes
                else {

					setValue(_element.getText());
				}
            }
        }
    }

    public void documentSaved(ReloadDocument doc) {}

}