/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.metadata.tableview;

import java.awt.*;
import javax.swing.*;

import uk.ac.reload.menu.*;
import uk.ac.reload.metadata.*;
import uk.ac.reload.widgets.*;

/**
 * A Metadata Table View Panel for the Metadata UI
 *
 * @author Phillip Beauvoir
 * @version 2003-07-31
 */
public class MetadataTablePanel
extends JPanel
{
    /**
     * The Treetable
     */
    protected MetadataTreeTable _treeTable;

    /**
     * The info Panel
     */
    protected ReloadInfoPanel _infoPanel;

    /**
     * ScrollPane
     */
    private JScrollPane _scrollPane;

    /**
     * Splitpane
     */
    private JSplitPane splitPane;

    /**
     * The Edit menu which we will use
     */
    protected Menu_Edit _editMenu;

    /**
     * Constructor
     * @param metadata The Metadata Model that we will View/Edit
     */
    public MetadataTablePanel(Menu_Edit editMenu) {
        _editMenu = editMenu;

        setLayout(new BorderLayout());

        // Create new ScrollPane
        _scrollPane = new JScrollPane();
        // Match background colour
        _scrollPane.getViewport().setBackground(UIManager.getColor("Table.background"));

        // Create new Info panel
        _infoPanel = new ReloadInfoPanel();

        // Set up a Split Pane
        splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitPane.setOneTouchExpandable(true);
        splitPane.setTopComponent(_scrollPane);
        splitPane.setBottomComponent(_infoPanel);
        add(splitPane, BorderLayout.CENTER);
    }

    /**
     * Set the view up to display the Metadata Document
     * @param metadata
     */
    public void setDocument(Metadata metadata) {
        // Set up a TreeTable and Model
        MetadataTreeTableModel model = new MetadataTreeTableModel(metadata);
        _treeTable = new MetadataTreeTable(model, _infoPanel, _editMenu);

        // Add it to the JScrollPane
        _scrollPane.setViewportView(_treeTable);

        // Select Root Node
        _treeTable.selectNode((MetadataTreeNode)model.getRoot());
    }

    /**
     * Clean up
     */
    public void destroy() {
        if(_treeTable != null) _treeTable.destroy();
        if(_infoPanel != null) _infoPanel.destroy();
    }

    /**
     * Refresh the View
     */
    public void refresh() {
        _treeTable.refresh();
    }

    /**
     * Called after Panel is shown so we can init stuff
     */
    public void initView() {
        splitPane.setDividerLocation(0.7);
        _infoPanel.initView();
    }

}
