package uk.ac.reload.metadata.vdexBrowser;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import java.io.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.util.*;
import uk.ac.reload.widgets.*;
import uk.ac.reload.vocabulary.*;


// public class MetadataVdexBrowser extends ReloadInternalFrame {
// Had to change from ReloadInternalFrame to JFrame because it would
// display with the MetadataEditorDialog of the CP.

public class MetadataVdexBrowser extends JFrame {

	private VdexVocabPanel vocabPanel;

	private VdexTermPanel termPanel;
	private VdexMetadataPanel mdPanel;

	private JTabbedPane tabPanel;

	private int selectedTab = 0;

	private VdexTreeModel treeModel;

	private VdexTree tree;


	public MetadataVdexBrowser(ReloadWidget source, String vdexSourceValue, String title) {

		// super(title, ReloadUIManager.ICON_VXBROWSE);
		super(title);
		setIconImage(ReloadUIManager.getIcon(ReloadUIManager.ICON_VXBROWSE).getImage());

		File file = this.getVdexDocument(vdexSourceValue);

		if(file != null) {

			ReloadDocument doc = ReloadDocumentHandler.openFile(file);

			if(doc != null) {

				if(doc instanceof Vocabulary) {

					Vocabulary vocabulary = (Vocabulary) doc;

					// create new vdex tree model
					treeModel = new VdexTreeModel(vocabulary);

					// create new tree
					tree = new VdexTree();

					// Add the treeModel to the tree
	    	    	tree.setModel(treeModel);
				}
			}
		}


		// NORTH
		vocabPanel = new VdexVocabPanel(treeModel.getVdexVocabNode());

		// CENTER
		termPanel = new VdexTermPanel(tree, source, this);
		tree.setTermPanel(termPanel);
		mdPanel = new VdexMetadataPanel(treeModel.getVdexVocabNode());

		tabPanel = new JTabbedPane();
		tabPanel.addTab(" Term ", null, termPanel, " Term ");
		tabPanel.addTab(" Metadata ", null, mdPanel, " Vocab Metadata ");
		tabPanel.addChangeListener(new tabListener());

		// ADD
		getContentPane().add(vocabPanel, BorderLayout.NORTH);
		getContentPane().add(tabPanel, BorderLayout.CENTER);


        // Dialog Size
        setSize(ReloadEditor.getInstance().getWidth() * 3 / 4, ReloadEditor.getInstance().getHeight() * 3 / 4);
        setLocationRelativeTo(ReloadEditor.getInstance());
	}



	public File getVdexDocument(String vdexSourceValue) {

		String VOCAB_ID = "vdex/vocabIdentifier";
		String VOCAB_NAME = "vdex/vocabName/langstring";

		DomPath domPath;

		boolean found = false;

        // Set up filter to only display xml files
        FileFilter filter = new FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return false;
                if(ReloadUtils.getFileExtension(file).equals("xml")) return true;
                return false;
            }
        };

		// Get the Files
		File[] files = ReloadDocumentHandler.folderMDVdexVocab.listFiles(filter);

        for(int i = 0; i < files.length; i++) {

			domPath = new DomPath(VOCAB_ID);

			found = XMLUtils.getElementByNameValuePair(Vocabulary.ROOT_NAME, domPath, files[i], vdexSourceValue);

			if(!found) {

				domPath = new DomPath(VOCAB_NAME);

				found = XMLUtils.getElementByNameValuePair(Vocabulary.ROOT_NAME, domPath, files[i], vdexSourceValue);
			}

			if(found) return files[i];
        }

		return null;
	}


	private class tabListener implements ChangeListener {

		public void stateChanged(ChangeEvent ce) {

			selectedTab = tabPanel.getSelectedIndex();

			tabPanel.setIconAt(selectedTab,null);
		}
	}
}