package uk.ac.reload.metadata.vdexBrowser;

import java.awt.*;
import java.util.Iterator;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.BadLocationException;

import org.jdom.*;

import uk.ac.reload.widgets.*;
import uk.ac.reload.ReloadEditor;
//import uk.ac.reload.ReloadUIManager;

public class VdexTermPanel extends JPanel {

	private JSplitPane splitPane;

	private JScrollPane left, right;

	private VdexTree _tree;

	private JPanel panel, panel2;

	private JButton select;

	private JEditorPane editor;

	private Element element;

	private String mimeType = "text/html";

	private ReloadWidget source;

	private MetadataVdexBrowser browser;


	public VdexTermPanel(VdexTree tree, ReloadWidget source, MetadataVdexBrowser browser) {

		super();

		_tree = tree;

		this.source = source;

		this.browser = browser;

		setLayout(new BorderLayout());

		// LEFT
		left = new JScrollPane(tree);

		// RIGHT
		panel = new JPanel();
		panel.setLayout(new BorderLayout());

		panel2 = new JPanel();
		((FlowLayout)panel2.getLayout()).setAlignment(FlowLayout.LEFT);
		panel2.setBackground(Color.white);
		select = new JButton("Select");
		select.addActionListener(new SelectListener());
		panel2.add(select);

		editor = new JEditorPane();
		editor.setContentType(mimeType);
		editor.setEditable(false);
		editor.addHyperlinkListener(new VdexHyperLinkListener());

		panel.add(panel2, BorderLayout.NORTH);
		panel.add(editor, BorderLayout.CENTER);

		right = new JScrollPane(panel);

		// ADD
		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT , left, right);
		splitPane.setOneTouchExpandable(true);
		splitPane.setDividerLocation(320);

		add(splitPane, BorderLayout.CENTER);
	}

	public void setElement(Element element) {

		this.element = element;

		this.updateEditor();
	}

	private void updateEditor() {

		String text = "";

		text += "<html><head>";

		//String cssFile = ReloadDocumentHandler.VDEX_BROWSER_STYLESHEET.getAbsolutePath();
		//System.out.println("cssFile = " + cssFile);
		//text += "<link rel=\"stylesheet\" type=\"text/css\" href=\"" + cssFile + "\">";

		text += "<style type=\"text/css\">";
		text += "body { font-size: 10px; font-family: sans-serif; }";
		text += "strong { font-size: 100%; text-align: left; } ";
		text += "tr { text-align: left; vertical-align: top; } ";
		text += "td.alter { background-color: #eeeeee }";
		text += "</style>";

		text += "</head><body>";

		text += "<table width=100% cellpadding=5>";

		text += "<tr>";
		text += "<td class=\"alter\" width=\"20%\">" + "<strong>Description</strong>" + "</td>";
		text += "<td class=\"alter\">" + this.getDescription() + "</td>";
		text += "</tr>";

		text += "<tr>";
		text += "<td>" + "<strong>Media Descriptor</strong>" + "</td>";
		text += "<td>" + this.getMediaDescriptor() + "</td>";
		text += "</tr>";

		text += "<tr>";
		text += "<td class=\"alter\">" + "<strong>Relationship</strong>" + "</td>";
		text += "<td class=\"alter\">" + this.getRelationship() + "</td>";
		text += "</tr>";

		text += "<tr>";
		text += "<td>" + "<strong>Metadata</strong>" + "</td>";
		text += "<td>" + this.getMetadata() + "</td>";
		text += "</tr>";

		text += "</table>";

		text += "</body></html>";


		editor.setText(text);
	}


	public String getDescription() {

		String TERM_DESCRIPTION = "description";
		String DESCRIPTION_LANGSTRING = "langstring";

		/*
		Element child = element.getChild(TERM_DESCRIPTION, element.getNamespace());

		String description = "";

		if(child != null) {

			String val = child.getChildTextTrim(DESCRIPTION_LANGSTRING, child.getNamespace());

			if(val != null && !val.equals("")) {

				description += val;
			}
			else {

				description += "Not available.";
			}
		}
		else {

			description += "Not available.";
		}

		return description;
		*/


		java.util.List children = element.getChildren(TERM_DESCRIPTION, element.getNamespace());

		String description = "", val="";

		Element child;

		if(children.size() == 0) {

			description += "Not available.";
		}
		else {
			for(int i = 0; i < children.size(); i++) {

    			child = (Element) children.get(i);

				if(child != null) {

					val = child.getChildTextTrim(DESCRIPTION_LANGSTRING, child.getNamespace());

					if(val != null && !val.equals("")) {

						if(i == 0) description += val;
						else description += "<br>" + val;
					}
					else {

						if(i == 0) description += "Not available.";
						else description += "<br>" + "Not available.";
					}
				}
				else {

					if(i == 0) description += "Not available.";
					else description += "<br>" + "Not available.";
				}
        	}
		}

		return description;
	}


	public String getMediaDescriptor() {

		String TERM_MEDIA_DESCRIPTOR = "mediaDescriptor";
		String MEDIA_DESCRIPTOR_LOCATOR = "mediaLocator";
		String MEDIA_DESCRIPTOR_INTERPRETATION_NOTE = "interpretationNote";
		String INTERPRETATION_NOTE_LANGSTRING = "langstring";

		/*
		Element child = element.getChild(TERM_MEDIA_DESCRIPTOR, element.getNamespace());

		String mediaDescriptor = "";

		if(child != null) {

			mediaDescriptor += "<i>" + "Media Locator: " + "</i>";

			String val = child.getChildTextTrim(MEDIA_DESCRIPTOR_LOCATOR, child.getNamespace());

			if(val != null && !val.equals("")) {

				mediaDescriptor += val + "<br>";
			}
			else {

				mediaDescriptor += "Not available." + "<br>";
			}


			mediaDescriptor += "<i>" + "Interpretation Note: " + "</i>";

			Element child2 = child.getChild(MEDIA_DESCRIPTOR_INTERPRETATION_NOTE, child.getNamespace());

			if(child2 != null) {

				val = child2.getChildTextTrim(INTERPRETATION_NOTE_LANGSTRING, child2.getNamespace());

				if(val != null && !val.equals("")) {

					mediaDescriptor += val;
				}
				else {

					mediaDescriptor += "Not available.";
				}
			}
			else {

				mediaDescriptor += "Not available.";
			}
		}
		else {

			mediaDescriptor += "<i>" + "Media Locator: " + "</i>" + "Not available." + "<br>";

			mediaDescriptor += "<i>" + "Interpretation Note: " + "</i>" + "Not available.";
		}

		return mediaDescriptor;
		*/

		java.util.List children = element.getChildren(TERM_MEDIA_DESCRIPTOR, element.getNamespace());

		String mediaDescriptor = "", val="";

		Element child;

		if(children.size() == 0) {

			mediaDescriptor += "Not available.";
		}
		else {
			for(int i = 0; i < children.size(); i++) {

    			child = (Element) children.get(i);

				if(child != null) {

					mediaDescriptor += "<i>" + "Media Locator: " + "</i>";

					val = child.getChildTextTrim(MEDIA_DESCRIPTOR_LOCATOR, child.getNamespace());

					if(val != null && !val.equals("")) {

						mediaDescriptor += val + "<br>";
					}
					else {

						mediaDescriptor += "Not available." + "<br>";
					}


					mediaDescriptor += "<i>" + "Interpretation Note: " + "</i>";

					Element child2 = child.getChild(MEDIA_DESCRIPTOR_INTERPRETATION_NOTE, child.getNamespace());

					if(child2 != null) {

						val = child2.getChildTextTrim(INTERPRETATION_NOTE_LANGSTRING, child2.getNamespace());

						if(val != null && !val.equals("")) {

							if(i < children.size()-1) mediaDescriptor += val + "<br>";
							else mediaDescriptor += val;
						}
						else {

							if(i < children.size()-1) mediaDescriptor += "Not available." + "<br>";
							mediaDescriptor += "Not available.";
						}
					}
					else {

						if(i < children.size()-1) mediaDescriptor += "Not available." + "<br>";
						mediaDescriptor += "Not available.";
					}
				}
				else {

					mediaDescriptor += "<i>" + "Media Locator: " + "</i>" + "Not available." + "<br>";

					if(i < children.size()-1) mediaDescriptor += "<i>" + "Interpretation Note: " + "</i>" + "Not available." + "<br>";
					else mediaDescriptor += "<i>" + "Interpretation Note: " + "</i>" + "Not available.";
				}

        	}
		}

		return mediaDescriptor;

	}


	public String getRelationship() {

		String TERM_RELATIONSHIP = "relationship";
		String RELATIONSHIP_TARGET = "targetTerm";
		String RELATIONSHIP_TYPE = "relationshipType";

		/*
		Element child = element.getChild(TERM_RELATIONSHIP, element.getNamespace());

		String relationship = "";

		if(child != null) {

			String val = child.getChildTextTrim(RELATIONSHIP_TYPE, child.getNamespace());

			if(val != null && !val.equals("")) {

				relationship += val + ": ";
			}
			else {

				relationship += "__: ";
			}

			val = child.getChildTextTrim(RELATIONSHIP_TARGET, child.getNamespace());

			if(val != null && !val.equals("")) {

				relationship += "<a href=\"relationship_target\">" + val + "</a>";
			}
			else {

				relationship += "__";
			}
		}
		else {

			relationship += "Not available.";
		}

		return relationship;
		*/

		java.util.List children = element.getChildren(TERM_RELATIONSHIP, element.getNamespace());

		String relationship = "", val="";

		Element child;

		if(children.size() == 0) {

			relationship += "Not available.";
		}
		else {
			for(int i = 0; i < children.size(); i++) {

    			child = (Element) children.get(i);

				if(child != null) {

					val = child.getChildTextTrim(RELATIONSHIP_TYPE, child.getNamespace());

					if(val != null && !val.equals("")) {

						relationship += val + ": ";
					}
					else {

						relationship += "__: ";
					}

					val = child.getChildTextTrim(RELATIONSHIP_TARGET, child.getNamespace());

					if(val != null && !val.equals("")) {

						if(i < children.size()-1) relationship += "<a href=\"relationship_target\">" + val + "</a>" + "<br>";
						else relationship += "<a href=\"relationship_target\">" + val + "</a>";
					}
					else {

						if(i < children.size()-1) relationship += "__" + "<br>";
						else relationship += "__";
					}

				}
				else {

					relationship += "__: ";

					if(i < children.size()-1) relationship += "__" + "<br>";
					else relationship += "__";
				}

        	}
		}

		return relationship;

	}


	public String getMetadata() {

		String TERM_METADATA = "metadata";

		/*
		String metadata = "";

		String val = element.getChildTextTrim(TERM_METADATA, element.getNamespace());

		if(val != null && !val.equals("")) {

			metadata += val;
		}
		else {

			metadata += "Not available.";
		}

		return metadata;
		*/


		java.util.List children = element.getChildren(TERM_METADATA, element.getNamespace());

		String metadata = "", val="";

		Element child;

		if(children.size() == 0) {

			metadata += "Not available.";
		}
		else {
			for(int i = 0; i < children.size(); i++) {

    			child = (Element) children.get(i);

				if(child != null) {

					// val = child.getText();

					Iterator contentList = child.getContent().iterator();

					while(contentList.hasNext()) {

						Object obj = contentList.next();

						if(obj instanceof Element) {

							Element objElem = (Element) obj;

							// System.out.println("Element obj = " + objElem.getName());
						}
						else if(obj instanceof Text) {

							Text objText = (Text) obj;

							// System.out.println("Text obj = " + objText.getTextTrim());
						}
						else if(obj instanceof Comment) {

							Comment objComment = (Comment) obj;

							// System.out.println("Comment obj = " + objComment.getText());


							// ASSUMPTION Metadata provided as Comment !!!!!!!!!!
							val = objComment.getText();
						}
						else if(obj instanceof CDATA) {

							CDATA objCData = (CDATA) obj;

							// System.out.println("CDATA obj = " + objCData.getText());
						}
					}


					if(val != null && !val.equals("")) {

						if(i == 0) metadata += val;
						else metadata += "<br>" + val;
					}
					else {

						if(i == 0) metadata += "Not available.";
						else metadata += "<br>" + "Not available.";
					}
				}
				else {

					if(i == 0) metadata += "Not available.";
					else metadata += "<br>" + "Not available.";
				}
        	}
		}

		return metadata;
	}



	protected class SelectListener implements ActionListener {

		public void actionPerformed(ActionEvent ae) {

			// get the selected id and caption from tree
			// send it back to the source

			if(source instanceof ReloadButtonTextField) {

				ReloadButtonTextField source1 = (ReloadButtonTextField) source;

				source1.setVdexValuesToDocument(_tree.getAllNodeNames());
			}
			else if (source instanceof ReloadButtonTextPane) {

				ReloadButtonTextPane source2 = (ReloadButtonTextPane) source;

				source2.setVdexValuesToDocument(_tree.getAllNodeNames());
			}
			else if(source instanceof ReloadButtonTextArea) {

				ReloadButtonTextArea source3 = (ReloadButtonTextArea) source;

				source3.setVdexValuesToDocument(_tree.getAllNodeNames());
			}

			// close the browse window

			browser.dispose();
		}
	}


	protected class VdexHyperLinkListener implements HyperlinkListener {

		public void hyperlinkUpdate(HyperlinkEvent e) {

			HyperlinkEvent.EventType type = e.getEventType();

			if (type == HyperlinkEvent.EventType.ACTIVATED) {

				if(e.getDescription().equals("relationship_target")) {

					// Highlight the selected Relationship Target on the Tree

					int startIndex = e.getSourceElement().getStartOffset();

					int endIndex = e.getSourceElement().getEndOffset();

					int length = endIndex - startIndex;

					try {

						String text = e.getSourceElement().getDocument().getText(startIndex,length);

						_tree.highlightNode(text.trim());
					}
					catch(BadLocationException ble) {

						if(ReloadEditor.DEBUG) System.out.println("BadLocationException when extracting text from document.");
					}
				}
			}
		}
	}
}



/*

	private void updateEditor() {

		String description = "<h2 style=\"font: arial;\" align=\"left\">" + "Description" + "</h2>";

		description += this.getDescription();

		String mediaDescriptor = "<h2 align=\"left\">" + "Media Descriptor" + "</h2>";

		mediaDescriptor += this.getMediaDescriptor();

		String relationship = "<h2 align=\"left\">" + "Relationship" + "</h2>";

		relationship += this.getRelationship();

		String metadata = "<h2 align=\"left\">" + "Metadata" + "</h2>";

		metadata += this.getMetadata();

		String text = description + "<br>" + mediaDescriptor + "<br>" + relationship + "<br>" + metadata;

		editor.setText(text);
	}


	public String getDescription() {

		String TERM_DESCRIPTION = "description";
		String DESCRIPTION_LANGSTRING = "langstring";

		Element child = element.getChild(TERM_DESCRIPTION, element.getNamespace());

		String description = "";

		if(child != null) {

			description += "<p>" + child.getChildTextTrim(DESCRIPTION_LANGSTRING, child.getNamespace()) + "</p>";
		}
		else {

			description += "<p>" + "Not available." + "</p>";
		}

		return description;
	}

	public String getMediaDescriptor() {

		String TERM_MEDIA_DESCRIPTOR = "mediaDescriptor";
		String MEDIA_DESCRIPTOR_LOCATOR = "mediaLocator";
		String MEDIA_DESCRIPTOR_INTERPRETATION_NOTE = "interpretationNote";
		String INTERPRETATION_NOTE_LANGSTRING = "langstring";

		Element child = element.getChild(TERM_MEDIA_DESCRIPTOR, element.getNamespace());

		String mediaDescriptor = "";

		if(child != null) {

			mediaDescriptor += "<p><b>" + "Media Locator: " + "</b>";

			String val = child.getChildTextTrim(MEDIA_DESCRIPTOR_LOCATOR, child.getNamespace());


			if(val != null) {

				mediaDescriptor += val + "</p>";
			}
			else {

				mediaDescriptor += "Not available." + "</p>";
			}


			mediaDescriptor += "<p><b>" + "Interpretation Note: " + "</b>";

			Element child2 = child.getChild(MEDIA_DESCRIPTOR_INTERPRETATION_NOTE, child.getNamespace());

			if(child2 != null) {

				mediaDescriptor += child2.getChildTextTrim(INTERPRETATION_NOTE_LANGSTRING, child2.getNamespace()) + "</p>";
			}
			else {

				mediaDescriptor += "Not available." + "</p>";
			}
		}
		else {

			mediaDescriptor += "<p><b>" + "Media Locator: " + "</b>" + "Not available." + "</p>";

			mediaDescriptor += "<p><b>" + "Interpretation Note: " + "</b>" + "Not available." + "</p>";
		}


		return mediaDescriptor;
	}


	public String getRelationship() {

		String TERM_RELATIONSHIP = "relationship";
		String RELATIONSHIP_TARGET = "targetTerm";
		String RELATIONSHIP_TYPE = "relationshipType";

		Element child = element.getChild(TERM_RELATIONSHIP, element.getNamespace());

		String relationship = "";

		if(child != null) {

			relationship += "<p><b>" + "Target Term: " + "</b>";

			String val = child.getChildTextTrim(RELATIONSHIP_TARGET, child.getNamespace());

			if(val != null) {

				relationship += "<a href=\"www.macs.hw.ac.uk\">" + val + "</a>" + "</p>";
			}
			else {

				relationship += "Not available." + "</p>";
			}

			relationship += "<p><b>" + "Relationship Type: " + "</b>";

			val = child.getChildTextTrim(RELATIONSHIP_TYPE, child.getNamespace());

			if(val != null) {

				relationship += val + "</p>";
			}
			else {

				relationship += "Not available." + "</p>";
			}
		}
		else {

			relationship += "<p><b>" + "Target Term: " + "</b>" + "Not available." + "</p>";

			relationship += "<p><b>" + "Relationship Type: " + "</b>" + "Not available." + "</p>";
		}

		return relationship;
	}


	public String getMetadata() {

		String TERM_METADATA = "metadata";

		String metadata = "";

		String val = element.getChildTextTrim(TERM_METADATA, element.getNamespace());

		if(val != null) {

			metadata += "<p>" + val + "</p>";
		}
		else {

			metadata += "<p>" + "Not available." + "</p>";
		}

		return metadata;
	}

*/





/*

	private void updateEditor() {

		String text = "";

		text += "<html><head></head><body>";

		text += "<table width=100% cellpadding=5>";

		text += "<tr align=\"left\" valign=\"top\">";
		text += "<td width=\"20%\" bgcolor=\"eeeeee\">" + "<font face=\"sans-serif\" size=3><b>Description</b></font>" + "</td>";
		text += "<td bgcolor=\"eeeeee\">" + this.getDescription() + "</td>";
		text += "</tr>";

		text += "<tr align=\"left\" valign=\"top\">";
		text += "<td>" + "<font face=\"sans-serif\" size=3><b>Media Descriptor</b></font>" + "</td>";
		text += "<td>" + this.getMediaDescriptor() + "</td>";
		text += "</tr>";

		text += "<tr align=\"left\" valign=\"top\">";
		text += "<td bgcolor=\"eeeeee\">" + "<font face=\"sans-serif\" size=3><b>Relationship</b></font>" + "</td>";
		text += "<td bgcolor=\"eeeeee\">" + this.getRelationship() + "</td>";
		text += "</tr>";

		text += "<tr align=\"left\" valign=\"top\">";
		text += "<td>" + "<font face=\"sans-serif\" size=3><b>Metadata</b></font>" + "</td>";
		text += "<td>" + this.getMetadata() + "</td>";
		text += "</tr>";

		text += "</table>";

		text += "</body></html>";


		editor.setText(text);
	}


	public String getDescription() {

		String TERM_DESCRIPTION = "description";
		String DESCRIPTION_LANGSTRING = "langstring";

		Element child = element.getChild(TERM_DESCRIPTION, element.getNamespace());

		String description = "<font face=\"sans-serif\" size=3>";

		if(child != null) {

			String val = child.getChildTextTrim(DESCRIPTION_LANGSTRING, child.getNamespace());

			if(val != null && !val.equals("")) {

				description += val;
			}
			else {

				description += "Not available.";
			}
		}
		else {

			description += "Not available.";
		}

		description += "</font>";

		return description;
	}

	public String getMediaDescriptor() {

		String TERM_MEDIA_DESCRIPTOR = "mediaDescriptor";
		String MEDIA_DESCRIPTOR_LOCATOR = "mediaLocator";
		String MEDIA_DESCRIPTOR_INTERPRETATION_NOTE = "interpretationNote";
		String INTERPRETATION_NOTE_LANGSTRING = "langstring";

		Element child = element.getChild(TERM_MEDIA_DESCRIPTOR, element.getNamespace());

		String mediaDescriptor = "<font face=\"sans-serif\" size=3>";

		if(child != null) {

			mediaDescriptor += "<i>" + "Media Locator: " + "</i>";

			String val = child.getChildTextTrim(MEDIA_DESCRIPTOR_LOCATOR, child.getNamespace());

			if(val != null && !val.equals("")) {

				mediaDescriptor += val + "<br>";
			}
			else {

				mediaDescriptor += "Not available." + "<br>";
			}


			mediaDescriptor += "<i>" + "Interpretation Note: " + "</i>";

			Element child2 = child.getChild(MEDIA_DESCRIPTOR_INTERPRETATION_NOTE, child.getNamespace());

			if(child2 != null) {

				val = child2.getChildTextTrim(INTERPRETATION_NOTE_LANGSTRING, child2.getNamespace());

				if(val != null && !val.equals("")) {

					mediaDescriptor += val;
				}
				else {

					mediaDescriptor += "Not available.";
				}
			}
			else {

				mediaDescriptor += "Not available.";
			}
		}
		else {

			mediaDescriptor += "<i>" + "Media Locator: " + "</i>" + "Not available." + "<br>";

			mediaDescriptor += "<i>" + "Interpretation Note: " + "</i>" + "Not available.";
		}

		mediaDescriptor += "</font>";

		return mediaDescriptor;
	}


	public String getRelationship() {

		String TERM_RELATIONSHIP = "relationship";
		String RELATIONSHIP_TARGET = "targetTerm";
		String RELATIONSHIP_TYPE = "relationshipType";

		Element child = element.getChild(TERM_RELATIONSHIP, element.getNamespace());

		String relationship = "<font face=\"sans-serif\" size=3>";

		if(child != null) {

			String val = child.getChildTextTrim(RELATIONSHIP_TYPE, child.getNamespace());

			if(val != null && !val.equals("")) {

				relationship += val + ": ";
			}
			else {

				relationship += "__: ";
			}

			val = child.getChildTextTrim(RELATIONSHIP_TARGET, child.getNamespace());

			if(val != null && !val.equals("")) {

				relationship += "<a href=\"relationship_target\">" + val + "</a>";
			}
			else {

				relationship += "__";
			}
		}
		else {

			relationship += "Not available.";
		}

		relationship += "</font>";

		return relationship;
	}


	public String getMetadata() {

		String TERM_METADATA = "metadata";

		String metadata = "<font face=\"sans-serif\" size=3>";

		String val = element.getChildTextTrim(TERM_METADATA, element.getNamespace());

		if(val != null && !val.equals("")) {

			metadata += val;
		}
		else {

			metadata += "Not available.";
		}

		metadata += "</font>";

		return metadata;
	}


*/