/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.prefs;

import javax.swing.*;

import uk.ac.reload.util.*;



/**
 * A Content Packaging Settings User Prefs Panel
 *
 * @author Phillip Beauvoir
 * @version 2003-07-31
 */
public class CP_PrefsPanel
extends PrefsPanel
{
    /**
     * Hide Resources check box
     */
    protected JCheckBox _checkbox_hideResources;

    /**
     * Hide SCORM Elements check box
     */
    protected JCheckBox _checkbox_hideSCORM;

    /**
     * Constructor
     */
    public CP_PrefsPanel() {
        setLayout(new XYLayout());
        constructFields();
    }

    /**
     * Set up the fields
     */
    private void constructFields() {
        int x = 10;
        int y = 10;

        // ============ Hide Resources ==================
        _checkbox_hideResources = new JCheckBox("Hide Resources node (effective the next time you open a Manifest)");
        add(_checkbox_hideResources, new XYConstraints(x, y, 0, CHECKBOX_HEIGHT));

        y += 20;

        // ============ Hide SCORM ==================
        _checkbox_hideSCORM = new JCheckBox("Hide SCORM nodes (effective the next time you open a Manifest)");
        add(_checkbox_hideSCORM, new XYConstraints(x, y, 0, CHECKBOX_HEIGHT));
    }

    /**
     * Update UserPrefs according to the controls' settings
     */
    public void saveToUserPrefs(UserPrefs prefs) {
        prefs.putBooleanValue(UserPrefs.CP_HIDE_RESOURCES, _checkbox_hideResources.isSelected());
        prefs.putBooleanValue(UserPrefs.CP_HIDE_SCORM, _checkbox_hideSCORM.isSelected());
    }

    /**
     * Set the controls' settings from those found in UserPrefs
     */
    public void setFields(UserPrefs prefs) {
        _checkbox_hideResources.setSelected(prefs.getBooleanValue(UserPrefs.CP_HIDE_RESOURCES));
        _checkbox_hideSCORM.setSelected(prefs.getBooleanValue(UserPrefs.CP_HIDE_SCORM));
    }

}
