/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.prefs;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import uk.ac.reload.*;

/**
 * The Preferences Dialog Window
 *
 * @author Phillip Beauvoir
 * @version 2003-07-10
 */
public class ReloadPrefsDialog
extends JDialog {

    /**
     * The General Prefs Panel
     */
    private General_PrefsPanel _generalPrefsPanel;

    /**
     * The Content Packager Prefs Panel
     */
    private CP_PrefsPanel _cpPrefsPanel;

    /**
     * The Metadata Editor Prefs Panel
     */
    private MD_PrefsPanel _mdPrefsPanel;

    /**
     * The Vocabulary Editor Prefs Panel
     */
    private VX_PrefsPanel _vxPrefsPanel;


    /**
     * The User Preferences
     */
    private UserPrefs _userPrefs;

    /**
     * Constructor
     */
    public ReloadPrefsDialog() {
        super(ReloadEditor.getInstance(), "Preferences", true);
        // Dialog Size
        setSize(500, 350);
        setLocationRelativeTo(ReloadEditor.getInstance());
        setResizable(false);

        // Make some panels
        _generalPrefsPanel = new General_PrefsPanel();
        _cpPrefsPanel = new CP_PrefsPanel();
        _mdPrefsPanel = new MD_PrefsPanel();
        _vxPrefsPanel = new VX_PrefsPanel();

        // Get the User Prefs from file
        _userPrefs = UserPrefs.getSharedInstance();

        // Tabbed Pane
        JTabbedPane tabPane = new JTabbedPane();
        tabPane.addTab("General", ReloadUIManager.getIcon(ReloadUIManager.ICON_APP16), _generalPrefsPanel);
        tabPane.addTab("Content Packager", ReloadUIManager.getIcon(ReloadUIManager.ICON_CP), _cpPrefsPanel);
        tabPane.addTab("Metadata Editor", ReloadUIManager.getIcon(ReloadUIManager.ICON_MD), _mdPrefsPanel);
        tabPane.addTab("Vocabulary Editor", ReloadUIManager.getIcon(ReloadUIManager.ICON_VX), _vxPrefsPanel);
        getContentPane().add(tabPane, BorderLayout.CENTER);

        // Button Panel
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));

        // OK Button
        JButton btnOK = new JButton("OK");
        btnOK.addActionListener(new OKClick());
        getRootPane().setDefaultButton(btnOK);
        buttonPanel.add(btnOK);

        // Cancel Buttton
        JButton btnCancel = new JButton("Cancel");
        btnCancel.addActionListener(new CancelClick());
        buttonPanel.add(btnCancel);

        // Add bottom Panel
        JPanel bottomPanel = new JPanel(new BorderLayout());
        bottomPanel.add(buttonPanel, BorderLayout.EAST);
        getContentPane().add(bottomPanel, BorderLayout.SOUTH);
    }

    /**
     * Over-ride this so we can set up the panels
     */
    public void show() {
        // Do this later because a modal dialog will block
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                // Update the Panels
                _generalPrefsPanel.setFields(_userPrefs);
                _cpPrefsPanel.setFields(_userPrefs);
                _mdPrefsPanel.setFields(_userPrefs);
                _vxPrefsPanel.setFields(_userPrefs);
            }
        });
        super.show();
    }

    /**
     * User pressed OK
     */
    protected void finish() {
        // Update the User Prefs
        _generalPrefsPanel.saveToUserPrefs(_userPrefs);
        _cpPrefsPanel.saveToUserPrefs(_userPrefs);
        _mdPrefsPanel.saveToUserPrefs(_userPrefs);
        _vxPrefsPanel.saveToUserPrefs(_userPrefs);

        try {
            _userPrefs.save();
        }
        catch(ReloadException ex) {
            ErrorDialogBox.showWarning("Could not save Preferences: ", "Preferences", ex);
        }

        dispose();
    }

    /**
     * OK handler
     */
    private class OKClick extends AbstractAction {
        public void actionPerformed(ActionEvent e) {
            finish();
        }
    }

    /**
     * Cancel handler
     */
    private class CancelClick extends AbstractAction {
        public void actionPerformed(ActionEvent e) {
            dispose();
        }
    }
}