/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.prefs.metadataprofiler;

import java.util.*;

import javax.swing.tree.*;

import uk.ac.reload.metadata.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.widgets.treetable.*;

/**
 * A TableModel for the Profile Editor.
 *
 * @author Phillip Beauvoir
 * @version 2003-03-10
 */
public class MetadataProfileTableModel
extends DefaultTreeModel
implements TreeTableModel
{

    /**
     * The Class types for the Columns - this is important because the tree
     * won't display properly without it.
     */
    static protected Class[] cTypes = { TreeTableModel.class, String.class, Boolean.class};

    static protected String[] cNames = {"Element", "Friendly Name", "Is Visible" };

    /**
     * The MetadataController for the Metadata Spec
     */
    MetadataSchemaController _controller;

    public MetadataProfileTableModel(MetadataSchemaController controller) {
        super(null);
        _controller = controller;
        // Create the root node
        root = new MetadataProfileTreeNode(controller.getReloadSchema().getRootElement());

        buildChildren((MetadataProfileTreeNode)root);
    }

    /**
     * Build up child nodes and add ReloadSchemaElement objects, so we'll add a new node to the model.
     * @param element The element that is to be added
     * @param node The tree node that contains it
     */
    public void buildChildren(MetadataProfileTreeNode node) {
        ReloadSchemaElement[] children = node.getSchemaElement().getChildren();
        for(int i = 0; i < children.length; i++) {
            ReloadSchemaElement child = children[i];
            // STOP RECURSION
            if(!child.hasAncestor(child)) {
                MetadataProfileTreeNode newNode = new MetadataProfileTreeNode(child);
                node.add(newNode);
                buildChildren(newNode);
            }
        }
    }

    public int getColumnCount() {
        return cNames.length;
    }

    /**
     * This asks for the value at column 1 or greater
     */
    public Object getValueAt(Object node, int column) {
        MetadataProfileTreeNode elementNode = (MetadataProfileTreeNode)node;
        switch(column) {
            case 1:
                return _controller.getElementFriendlyName(elementNode.getSchemaElement().getDomPath());
            case 2:
                return new Boolean(true);
            default:
                return elementNode.getSchemaElement();
        }
    }

    public String getColumnName(int c) {
        return cNames[c];
    }

    public Class getColumnClass(int c) {
        return cTypes[c];
    }

    public boolean isCellEditable(Object node, int column) {
        return true;
    }

    public Object getChild(Object parent, int index) {
        MetadataProfileTreeNode elementNode = (MetadataProfileTreeNode) parent;
        return elementNode.getChildAt(index);
    }

    public int getChildCount(Object parent) {
        MetadataProfileTreeNode elementNode = (MetadataProfileTreeNode) parent;
        return elementNode.getChildCount();
    }

    public void setValueAt(Object aValue, Object node, int column) {

    }

}