/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.prefs.vocabularyprofiler;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import uk.ac.reload.*;
import uk.ac.reload.vocabulary.*;
import uk.ac.reload.widgets.*;

/**
 * The Vocabulary Profile Editor Frame
 * !!!!!!! Unfinished  !!!
 *
*/
public class VocabularyProfileEditorFrame
extends ReloadInternalFrame
{

    ReloadComboBox profileCombobox;

    VocabularySchemaController vxSchemaController;


    /**
     * Default Constructor
     */
    public VocabularyProfileEditorFrame() {
        super("Vocabulary Profile Editor", ReloadUIManager.ICON_VX);
        // Create the view
        createView();
    }


    protected void createView() {
        // Get our UI Controller
        vxSchemaController = VocabularySchemaController.getDefaultSchemaController();

        // Get a new Table
        VocabularyProfileTableModel tablemodel = new VocabularyProfileTableModel(vxSchemaController);
        VocabularyProfilerTreeTable treeTable = new VocabularyProfilerTreeTable(tablemodel);

        // Add it to a JScrollPane
        JScrollPane sp1 = new JScrollPane(treeTable);
        // Match background colour
        sp1.getViewport().setBackground(UIManager.getColor("Table.background"));

        // Textfield
        JPanel panelText = new JPanel();
        panelText.setLayout(new BoxLayout(panelText, BoxLayout.X_AXIS));
        panelText.add(new JLabel(" Save as template: "));
        panelText.add(new JTextField());

        // Combobox
        JPanel panelCombo = new JPanel();
        panelCombo.setLayout(new BoxLayout(panelCombo, BoxLayout.X_AXIS));
        panelCombo.add(new JLabel("           Profile: "));
        VocabularyProfile[] profiles = VocabularyProfile.getVocabularyProfiles();
        profileCombobox = new ReloadComboBox(profiles);
        panelCombo.add(profileCombobox);

        // Listen to changes on the Profile Combo
        profileCombobox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				VocabularyProfile profile = (VocabularyProfile)profileCombobox.getSelectedItem();

				loadSchema(profile);
                updateView();
            }
        });


        JPanel fieldPanel = new JPanel(new GridLayout(1, 2));
        fieldPanel.add(panelText);
        fieldPanel.add(panelCombo);

        getContentPane().add(fieldPanel, BorderLayout.NORTH);
        getContentPane().add(sp1, BorderLayout.CENTER);
    }

    /**
     * Dynamically load a Vocabulary Schema
     */
    public VocabularySchemaController loadSchema(VocabularyProfile vxProfile) {

        vxSchemaController = vxSchemaController.setVocabularySchema(vxProfile);

        return vxSchemaController;
    }

    /**
     * Update VocabularyProfileEditorFrame
     */
    public void updateView() {

        // Get a new Table
        VocabularyProfileTableModel tablemodel = new VocabularyProfileTableModel(vxSchemaController);
        VocabularyProfilerTreeTable treeTable = new VocabularyProfilerTreeTable(tablemodel);

        // Add it to a JScrollPane
        JScrollPane sp1 = new JScrollPane(treeTable);
        // Match background colour
        sp1.getViewport().setBackground(UIManager.getColor("Table.background"));

        getContentPane().add(sp1, BorderLayout.CENTER);
    }
}
