/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.schema;

import java.util.*;
import java.io.*;
import javax.swing.*;

import org.jdom.Element;

import uk.ac.reload.*;
import uk.ac.reload.util.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.vocabulary.*;

/**
 * A Reload Vocabulary List - A String list and default value
 *
 * @author Phillip Beauvoir
 * @version 2003-05-14
 */
public class ReloadVocabularyList {
    protected static String INTERNAL_LISTNAME = "internal";
    protected static String LANG_LISTNAME = "lang";
    protected static String YESNO_LISTNAME = "yesno";
    protected static String TRUEFALSE_LISTNAME = "truefalse";
	/**
	* Tina Manoharan
	* 2003-09-09
	*/
	protected static String VDEX_VOCAB = "vdexVocab";
	protected static String DEFAULT_VDEX_VOCAB = "default_vdexVocab";
	protected static String VOCAB_ID = "vdex/vocabIdentifier";
	protected static String VOCAB_NAME = "vdex/vocabName/langstring";
	protected static String TERM_CAPTION = "vdex/term/caption/langstring";

    /**
     * Yes / No list
     */
    protected static String[] YESNO_LIST = new String[] { "yes", "no" };

    /**
     * True / False vocabulary list
     */
    protected static String[] TF_LIST = new String[] { "true", "false" };

    /**
     * The path mapping
     */
    protected DomPath domPath;

    /**
     * The list name.  If the list comes from the Schema, this should be set to "internal"
     * or can be set to one of the special list names like "lang" or "yesno"
     */
    protected String listName;

    protected String defaultListName;

    /**
     * The Actual String list, initialised to an Empty list
     */
    protected String[] list = new String[] { "" };

    /**
     * The Default String list, initialised to an Empty list
     */
    protected String[] defaultList = new String[] { "" };

    /**
     * The default value
     */
    protected String defaultValue;


    public ReloadVocabularyList() {

    }

    public ReloadVocabularyList(DomPath domPath, String listName) {
        this.domPath = domPath;
        this.listName = listName;
    }

    public DomPath getDomPath() {
        return domPath;
    }

    /**
     * Set the Vocabulary List
     * @param list
     */
    public void setList(String[] list) {
        this.list = list;
    }

    /**
     * Set the Default Vocabulary List
     * @param list
     */
    public void setDefaultList(String[] list) {
        this.defaultList = list;
    }

    /**
     * Get the Vocab list.  It will at least be an array of one empty String
     * @return
     */
    public String[] getList() {
        // Special case of lang list
        if(listName.equals(LANG_LISTNAME)) {

			return getLanguageList();
		}
        // Special case of yesno list
        if(listName.equals(YESNO_LISTNAME)) {

			return YESNO_LIST;
		}
        // Special case of truefalse list
        if(listName.equals(TRUEFALSE_LISTNAME)) {

			return TF_LIST;
		}

		/**
		* Tina Manoharan
		* 2003-09-09
		*/
		// Special case of Vdex Vocab list
		if(listName.equals(VDEX_VOCAB) || listName.equals(DEFAULT_VDEX_VOCAB)) {

			return getVdexVocabList();
		}

        return list;
    }

    // (5) getList(String vocabSource, Element element)
    // (6) if vocabSource == defaultListName get defaultList else get List from vdexFile
    // (7) update the list and inform listeners MetadataComboBox & ReloadListBox that are related to this element instance
    // if(element == _reloadDocument.getElement(_schemaElement))
    // (8) created a defaultList

    /**
     * Set the Default Vocabulary List
     * @param list
     */
    public void setListUsingVocabSource(Object source, String vocabSource, Element valueElement, DomPath domPath) {

		// (9) get list items from vocabSource vdex file.
		// (10) inform listeners that have that particular valueElement and domPath

		if(!vocabSource.equals("") && !vocabSource.equals(defaultListName) && vocabSource != null) {

			this.list = getListFromVdexSource(vocabSource, TERM_CAPTION);
		}
		else {

			this.list = this.defaultList;
		}

        ReloadVocabularyListenerEvent event = new ReloadVocabularyListenerEvent(source, ReloadVocabularyList.this, this.list);
        fireElementChanged(event);
    }


    public String[] getListFromVdexSource(String vocabSource, String elementName) {

		// (11) Find the xml file that has this vocabID or vocabName
		// (12) Get all the term captions

		ArrayList result = new ArrayList();

		DomPath domPath;

		File stylesheet = null;

        // Set up filter to only display xml files
        FileFilter filter = new FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return false;
                if(ReloadUtils.getFileExtension(file).equals("xml")) return true;
                return false;
            }
        };

        // Get the File with the given vocabName/ID
        File[] files = ReloadDocumentHandler.folderMDVdexVocab.listFiles(filter);
		boolean found = false;

        for(int i = 0; i < files.length; i++) {

			domPath = new DomPath(VOCAB_ID);

			found = XMLUtils.getFileByNameValuePair(Vocabulary.ROOT_NAME, domPath, files[i], vocabSource);

			if(!found) {

				domPath = new DomPath(VOCAB_NAME);

				found = XMLUtils.getFileByNameValuePair(Vocabulary.ROOT_NAME, domPath, files[i], vocabSource);
			}

			if(found) {

				if(elementName.equals(TERM_CAPTION)) {

					stylesheet = ReloadDocumentHandler.VDEX_TO_TEMP_STYLESHEET;
				}

				result = XMLUtils.getElementValuesFromFileByName(files[i], stylesheet);

				break;
			}
        }

		return (String[]) result.toArray(new String[result.size()]);
	}

    public String getFirstListItem() {

    	String[] wholeList = this.getList();

    	return wholeList[0];
	}


    /**
     * Get the Name of the List
     * @return
     */
    public String getListName() {
        return listName;
    }


    /**
     * Set the default listName for this vocab list
     * @param defaultListName
     */
    public void setDefaultListName(String defaultListName) {

        this.defaultListName = defaultListName;

		if(listName.equals(DEFAULT_VDEX_VOCAB) && this.defaultListName != null) this.defaultValue = this.defaultListName;
    }

    /**
     * Return true if this is a language list
     * @return
     */
    public boolean isLangList() {
        return listName.equals(LANG_LISTNAME);
    }

    /**
     * Set the default value for this vocab list
     * @param defaultValue
     */
    public void setDefaultValue(String defaultValue) {

        this.defaultValue = defaultValue;
    }

    /**
     * Get the Default Value
     * @return The default value for this vocab list
     */
    public String getDefaultValue() {

        return defaultValue;
    }

    /**
     * Get a Vocabulary List that is full of the Language Strings
     * @return
     */
    private String[] getLanguageList() {
        return Locale.getISOLanguages();
    }

	/**
	* Tina Manoharan
	* 2003-09-09
	*/
    /**
     * Get Vdex Vocab List from saved files in helpers/md/vdexVocab
     * @return
     */
    private String[] getVdexVocabList() {

		DomPath domPath;

        // Set up filter to only display xml files
        FileFilter filter = new FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return false;
                if(ReloadUtils.getFileExtension(file).equals("xml")) return true;
                return false;
            }
        };

        // Get the Files
        File[] files = ReloadDocumentHandler.folderMDVdexVocab.listFiles(filter);
        String s;
        Vector v = new Vector();

        if(listName.equals(DEFAULT_VDEX_VOCAB)) v.addElement(defaultListName);

        for(int i = 0; i < files.length; i++) {

			domPath = new DomPath(VOCAB_ID);

			s = XMLUtils.getElementValueByName(Vocabulary.ROOT_NAME, domPath, files[i]);

			if(s == null || s.equals("")) {

				domPath = new DomPath(VOCAB_NAME);

				s = XMLUtils.getElementValueByName(Vocabulary.ROOT_NAME, domPath, files[i]);
			}

			if(s != null && !s.equals("")) {

				v.addElement(s);
			}
        }

        String[] vdexVocabList = new String[v.size()];
        v.copyInto(vdexVocabList);

        return vdexVocabList;
    }


    //============================== LISTENER EVENTS  ==========================

    /**
     * List of Listeners
     */
    private Vector _listeners = new Vector();

    /**
     * Add a ReloadVocabularyListener
     * @param listener The ReloadVocabularyListener
     */
    public synchronized void addReloadVocabularyListener(ReloadVocabularyListener listener) {
        if(!_listeners.contains(listener)) _listeners.addElement(listener);
    }

    /**
     * Remove a ReloadVocabularyListener
     * @param listener The ReloadVocabularyListener
     */
    public synchronized void removeReloadVocabularyListener(ReloadVocabularyListener listener) {
        _listeners.removeElement(listener);
    }

    /**
     * Tell our listeners that we have changed the list
     */
    public void fireElementChanged(ReloadVocabularyListenerEvent event) {

        for(int i = _listeners.size() - 1; i >= 0; i--) {
            ReloadVocabularyListener listener = (ReloadVocabularyListener)_listeners.elementAt(i);
            listener.vocabularyListChanged(event);
        }
	}
}