/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.util;

import java.io.*;

import javax.swing.*;

import uk.ac.reload.prefs.*;

/**
 * An implementation of JFileChooser that remembers the last directory.
 *
 * @author Phillip Beauvoir
 * @version 2003-07-10
 */
public class ReloadFileChooser
extends JFileChooser
{
    /**
     * The initial default directory
     */
    protected static File lastFolder;

    /**
     * Try and set the initial folder to the one in user prefs
     */
    static {
        if(lastFolder == null) {
            String openFolder = UserPrefs.getSharedInstance().getValue(UserPrefs.GENERAL_PREFS_DEFAULT_FOLDER);
            if(openFolder != null) {
                lastFolder = new File(openFolder);
                if(!lastFolder.exists()) {
                    lastFolder = new File(System.getProperty("user.dir"));
                }
            }
            else lastFolder = new File(System.getProperty("user.dir"));
        }
    }

    public ReloadFileChooser() {
        this(lastFolder);
    }

    public ReloadFileChooser(File folder) {
        super(folder);
    }

    /**
    * Returns the selected file and updates the local dir
    * @return The selected file
    */
    public File getSelectedFileAndStore() {
        File file = getSelectedFile();
        if(file != null) lastFolder = file.getParentFile();
        return file;
    }

    /**
    * Returns the selected files and updates the local dir
    * @return The selected files
    */
    public File[] getSelectedFilesAndStore() {
        File[] files = getSelectedFiles();
        if(files != null) lastFolder = files[0].getParentFile();
        return files;
    }

    /**
    * Store the current directory.
    * @param dir The directory to set.
    */
    public void setStoredFolder(File folder) {
        lastFolder = folder;
    }

    public File getStoredFolder() {
        return lastFolder;
    }
}
