/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.vocabulary;

import java.util.*;

import org.jdom.*;
import uk.ac.reload.dom.*;

/**
 * A Vocabulary Profile Element
 *
 */
public class VocabularyProfileElement {
    /**
     * The Path of the Element
     */
    protected DomPath domPath;

    /**
     * The name to display for this Element
     */
    protected String name;

    /**
     * The Parent, or null if no parent
     */
    protected VocabularyProfileElement parent;

    /**
     * This Element's children
     */
    protected VocabularyProfileElement[] children;

    protected boolean isGroup;

    public VocabularyProfileElement(Element element) {
        name = element.getAttributeValue("name");
        domPath = new DomPath(element.getAttributeValue("path"));
        isGroup = element.getName().equals("group");
        children = makeChildren(element);
    }

    /**
     * Get the path for this Profile Element
     * @return
     */
    public DomPath getDomPath() {
        return domPath;
    }

    /**
     * Get the name to display for this Element
     * @return
     */
    public String getName() {
        return name;
    }

    /**
     * Find out if this Element is merely a container for others
     * @return
     */
    public boolean isGroup() {
        return isGroup;
    }

    /**
     * Return the Children of this Profile Element
     * @return
     */
    public VocabularyProfileElement[] getChildren() {
        return children;
    }

    /**
     * Get the parent
     * @return
     */
    public VocabularyProfileElement getParent() {
        return parent;
    }

    /**
     * Wrap each child Element in a VocabularyProfileElement
     * @param element
     * @return
     */
    private VocabularyProfileElement[] makeChildren(Element element) {
        Vector v = new Vector();

        // Get all "element" children
        Iterator elementlist = element.getChildren().iterator();

        // Enumerate through them
        while(elementlist.hasNext()) {
            // Get the Element
            Element child = (Element) elementlist.next();
            // New VocabularyProfileElement
            VocabularyProfileElement vxProfileChildElement = new VocabularyProfileElement(child);
            vxProfileChildElement.parent = this;
            v.addElement(vxProfileChildElement);
        }

        VocabularyProfileElement[] array = new VocabularyProfileElement[v.size()];
        v.copyInto(array);
        return array;
    }
}