/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.vocabulary.formview;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

import uk.ac.reload.vocabulary.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.widgets.*;


/**
 * A Text Area widget.
 *
 */
public class VocabularyTextField
extends VocabularyField
implements DocumentListener
{
    /**
     * An internal flag
     */
    boolean _allowNotification = true;

    /**
     * The suitable Component we shall use
     */
    protected ReloadWidget _reloadWidget;

    /**
     * Constructor
     */
    public VocabularyTextField(Vocabulary vocabulary, ReloadSchemaElement schemaElement) {
        super(vocabulary, schemaElement);

        // Determine type of text field
        _reloadWidget = getVocabularyTextFieldWidget(schemaElement);
        ((JComponent)_reloadWidget).setBorder(null);

        _element = vocabulary.getElement(_schemaElement);
        if(_element != null) setValue(_element.getText());

        // We listen to Text Changes
        _reloadWidget.addDocumentListener(this);

        // We listen to Element changes
        vocabulary.addReloadDocumentListener(this);
    }

    /**
     * @return A Text Widget suitable for the VX Form display
     */
    protected ReloadWidget getVocabularyTextFieldWidget(ReloadSchemaElement schemaElement) {
        if(schemaElement == null) {

			return getDefaultWidget();
		}

        ReloadWidget component = null;

        ReloadSchemaController schemaController = _reloadDocument.getReloadSchemaController();

        String widget_type = schemaController.getWidgetType(schemaElement.getDomPath());

        if(widget_type != null) {
            if(widget_type.equals("textpane")) {

                component = new ReloadTextArea();
                ((ReloadTextArea)component).setColumns(50);
            }
            else if(widget_type.equals("numberfield")) {

                component = new ReloadNumberField();
                ((ReloadNumberField)component).setColumns(30);
            }
            else {
				component = getDefaultWidget();
			}
        }
        else {
			component = getDefaultWidget();
		}

        // Set Max String length
        int maxLength = -1;
        String value = schemaController.getFacetValue(schemaElement, "maxLength");
        if(value != null) maxLength = Integer.parseInt(value);
        component.setMaxLength(maxLength);

        return component;
    }

    /**
     * @return a default widget
     */
    private ReloadWidget getDefaultWidget() {
        ReloadTextField textField = new ReloadTextField();
        textField.setColumns(50);
        return textField;
    }

    /**
     * Clean up
     */
    public void destroy() {
        _reloadWidget.removeDocumentListener(this);
        _reloadDocument.removeReloadDocumentListener(this);
    }

    /**
     * Get the value of this Widget
     * @return The value contained in the Widget
     */
    public String getValue() {
        return _reloadWidget.getTextValue();
    }

    /**
     * Set the value of the Widget
     * @param value The value
     */
    public void setValue(String value) {
        _allowNotification = false;
        _reloadWidget.setTextValue(value);
        _allowNotification = true;
    }

    /**
     * @return the Component that we're using
     */
    public Component getComponent() {

        return (Component)_reloadWidget;
    }

// =============================================================================
// === These events are fired from the JTextArea when the textfield changes ====
// =============================================================================

    public void removeUpdate(DocumentEvent evt) {
        if(_allowNotification) fireElementChanged();
    }

    public void changedUpdate(DocumentEvent evt) {
        if(_allowNotification) fireElementChanged();
    }

    public void insertUpdate(DocumentEvent evt) {
        if(_allowNotification) fireElementChanged();
    }

}