/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.widgets;

import java.awt.*;
import javax.swing.*;

import uk.ac.reload.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.schema.*;


/**
 * A Panel to display the tip
 *
 * @author Phillip Beauvoir
 * @version 2003-07-31
 */
public class ReloadTipPanel extends JPanel {
    /**
     * The JLabel to display the friendly name for the Element
     */
    protected JLabel _nameLabel;

    /**
     * The Textpane to display the tip
     */
    protected JLabel _tipLabel;

    /**
     * Constructor
     */
    public ReloadTipPanel() {
        setLayout(new BorderLayout());
        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        // Name label
        _nameLabel = new JLabel();
        _nameLabel.setForeground(Color.BLUE);
        _nameLabel.setFont(ReloadUIManager.boldFont12);
        _nameLabel.setBorder(BorderFactory.createEtchedBorder());
        add(_nameLabel, BorderLayout.NORTH);

        _tipLabel = new JLabel();
        _tipLabel.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));
        _tipLabel.setVerticalAlignment(SwingConstants.TOP);

        add(_tipLabel, BorderLayout.CENTER);
    }

    /**
     * Clear the Settings
     */
    public void clear() {
        _nameLabel.setText("");
        _tipLabel.setText("");
    }

    /**
     * Update the text tip panel
     */
    public void setTip(ReloadSchemaController schemaController, ReloadSchemaElement schemaElement) {
        if(schemaElement != null) {
            DomPath domPath = schemaElement.getDomPath();

            // Friendly Name
            String fname = schemaController.getElementFriendlyName(domPath);
            if(fname == null) fname = schemaElement.getName();
            _nameLabel.setText(" " + fname);

            // Tip
            String tip = schemaController.getElementTip(domPath);
            if(tip == null) _tipLabel.setText("");
            else _tipLabel.setText("<html>" + "<font color=blue>" + tip + "</font><br>");
        }
        else {
            _nameLabel.setText("");
            _tipLabel.setText("");
        }
    }

}
